<?php
namespace Notifal\backend;
use Notifal\functions;
use Notifal\maintenance;
use Notifal\templates as templates;

/**
 * backend menu
 *
 * @version 1.0.0
 *
 */
if ( ! defined( 'ABSPATH' ) ) exit;


Class menu{
    use functions;

    /**
     * Menu constructor
     * @since 1.0.0
     */
    public function __construct(){
        $bansename= Notifal_BASE;
        add_filter( "plugin_action_links_$bansename", [$this,'add_extra_link'] );
        add_action('admin_menu',[$this,'creat_menu']);
        add_action('adminmenu', [$this,'add_custom_class_to_submenu']);
    }

    /**
     * Creat Backend Menu
     *
     * @since 1.0.0
     * @return void
     */
    public function creat_menu(): void
    {
        add_menu_page('Notifal',
            'Notifal',
            'manage_woocommerce',
            'Notifal-dashboard',
            '',
            $this->get_image('menu-icon.png'),
            10);

        add_submenu_page('Notifal-dashboard',
            esc_html__('Dashboard',Notifal_DOMAIN) ,
            esc_html__('Dashboard',Notifal_DOMAIN),
            'manage_woocommerce',
            'Notifal-dashboard',
            [$this,'dashboard_content_callback'],
            1);

        add_submenu_page('Notifal-dashboard',
            esc_html__('Add new notif',Notifal_DOMAIN) ,
            esc_html__('Add new',Notifal_DOMAIN),
            'manage_woocommerce',
            'notif',
            [$this,'add_new_callback'],
            2);

        add_submenu_page('Notifal-dashboard',
            esc_html__('Notifs',Notifal_DOMAIN) ,
            esc_html__('Notifs',Notifal_DOMAIN),
            'manage_woocommerce',
            'notifs-list',
            [$this,'notifications_callback'],
            3);

        if(!maintenance::is_pro_installed()){
            add_submenu_page('Notifal-dashboard',
                esc_html__('Upgrade',Notifal_DOMAIN) ,
                esc_html__('Upgrade',Notifal_DOMAIN),
                'manage_woocommerce',
                'Notifal-upgrade',
                [$this,'license_callback'],
                4);
        }


        if(maintenance::is_pro_active_but_not_connected()){
            add_submenu_page('Notifal-dashboard',
                esc_html__('Connect And Activate',Notifal_DOMAIN) ,
                esc_html__('Connect And Activate',Notifal_DOMAIN),
                'manage_woocommerce',
                'Notifal-Pro-connect',
                [$this,'license_callback'],
                5);
        }

    }

    /**
     * Callback view of home slug
     *
     * @since 1.0.0
     * @return void
     */
    public function add_new_callback(): void
    {
        $functions = $this;
        include (templates::get_backend_template_path('single-notif.php'));
    }

    /**
     * notifications list content callback
     * @since 1.0.0
     * @return void
     */
    public function notifications_callback(): void
    {
        $functions = $this;
        include (templates::get_backend_template_path('notifs.php'));
    }

    /**
     * license content callback
     * @since 1.0.0
     * @return void
     */
    public function license_callback(): void
    {
        include (templates::get_backend_template_path('license.php'));
    }


    /**
     * Add menu to plugins list menu
     *
     * @since 1.0.0
     * @param $links
     * @return mixed
     */
    public function add_extra_link($links){

        $docs = '<a target="_blank"  href="https://notifal.com/category/woonotif/documentation/">'.esc_html__('Docs',Notifal_DOMAIN).'</a>';
        $support = '<a target="_blank"  href="https://notifal.com/">'.esc_html__('Support',Notifal_DOMAIN).'</a>';
        if(!maintenance::is_pro_installed()){
            $upgrade = '<a target="_blank" style="color: #93003f;font-weight: 700;" href="https://notifal.com/pricing/">'.esc_html__('Upgrade',Notifal_DOMAIN).'</a>';
            $links[] = $upgrade;
        }
        $links[] = $support;
        $links[] = $docs;
        return $links;
    }


    /**
     * Add class to upgrade admin submenu
     * @since 1.0.0
     * @return void
     */
    public function add_custom_class_to_submenu(){
        ?>
        <script>
            document.addEventListener('DOMContentLoaded', function () {
                const submenuItem = document.querySelector('#toplevel_page_Notifal-dashboard .wp-submenu a[href="admin.php?page=Notifal-upgrade"]');
                if (submenuItem) {
                    submenuItem.classList.add('important-Notifal-btn'); // Add a class
                }
            });
        </script>
        <?php
    }


    public function dashboard_content_callback(){
        $functions = $this;
        include (templates::get_backend_template_path('dashboard.php'));
    }
}