<?php
namespace Notifal;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

/**
 * Notifal features
 *
 * Notifal feature handles the features of Notifal
 *
 * @since 1.0.0
 */

class features {
    use functions;

    public function __construct(){
        add_action('Notifal_before_notif_saved',[$this,'check_active_notifs'],10,3);
        add_filter('Notifal_notif_products_before_saved',[$this,'check_products'],10,1);
        add_filter('Notifal_notif_conditions_before_saved',[$this,'check_conditions'],10,1);
        add_filter('before_update_status',[$this,'update_status'],10,2);
    }

    /**
     * check if there are more than one active notif if pro is not installed.
     *
     * @since 1.0.0
     * @param $output_data
     * @param $status
     * @param $id
     * @return void
     */
    public function check_active_notifs($output_data,$status,$id){
        if(!maintenance::is_pro_active()){
            if($status=='Active'){
                $notif_table =notif::$notifal_table;
                $active_notifs = db::get_table_rows($notif_table,'id',["status='Active'"]);
                if(count($active_notifs)>=1){
                    if(!empty($id)){
                        if($id===$active_notifs[0]->id) return;
                    }
                    $output_data['message']='More than one active';
                    print_r(json_encode($output_data));
                    exit();
                }
            }
        }
    }

    /**
     * Prevent selected product and categories while active is not pro
     *
     * @since 1.0.0
     * @param $products
     * @return mixed
     */
    public function check_products($products){
        if(!maintenance::is_pro_active()){
            if(in_array($products['notif_products'],['all-products-except','specific-products'])){
                $products['notif_products']='all-products';
                $products['selected_products']=[];
                $products['selected_categories']=[];
            }
        }
        return $products;
    }

    /**
     * Prevent selected page and categories while active is not pro
     *
     * @since 1.0.0
     * @param $products
     * @return mixed
     */
    public function check_conditions($conditions){
        if(!maintenance::is_pro_active()){
            if(in_array($conditions['page_type'],['all-pages-except','specific-pages'])){
                $conditions['page_type']='all-pages';
                $conditions['selected_pages']=[];
                $conditions['selected_category_pages']=[];
            }
        }
        return $conditions;
    }


    /**
     * Prevent ti update notif status to Active while pro is not installed.
     *
     * @since 1.0.0
     * @param $update_data
     * @param $id
     * @return mixed|string[]
     */
    public function update_status($update_data,$id){
        if(!maintenance::is_pro_active()){
            $notif_table =notif::$notifal_table;
            $active_notifs = db::get_table_rows($notif_table,'id',["status='Active'"]);
            if(count($active_notifs)>=1){
                if($id!==$active_notifs[0]->id) {
                    $update_data=['status'=>'Inactive'];
                }
            }
        }
        return $update_data;
    }


}