<?php
namespace Notifal\notif;
use Notifal\db;
use Notifal\notif;
if (session_status() === PHP_SESSION_NONE) {
    session_start();
}
/**
 * Analytics handler
 *
 * @since 1.1.0
 * @version 1.0.0
 */
if ( ! defined( 'ABSPATH' ) ) exit;

class analytics
{

    public function __construct()
    {
        add_action('wp',[$this,'handle_click_action']);
        add_action('woocommerce_new_order', [$this,'save_order_meta'],10,2);
        add_action('add_meta_boxes', [$this,'add_metabox_to_order']);
    }


    /**
     * Handles the "click" action in the URL, updating the today user actions table.
     *
     * This function checks if the URL contains "action_notif=click" and a valid "notif_id".
     * If found, it calls `add_current_user_data_action` to increment or initialize the "click" action.
     *
     * @since 1.1.0
     *
     * @return void
     */
    public static function handle_click_action() {
        if (isset($_GET['action_notif'], $_GET['notif_id']) && $_GET['action_notif'] === 'click') {
            $notif_id = intval($_GET['notif_id']);
            $notif_action_position = intval($_GET['notif_action_position']);
            if ($notif_id <= 0) {
                return;
            }

            do_action('notifal_before_save_click_session',$notif_id,$notif_action_position);
            $url = remove_query_arg(['action_notif', 'notif_id','notif_action_position']);
            if(isset($_SESSION['Notifal_click_notif_id'])){
                if(!in_array($notif_id,$_SESSION['Notifal_click_notif_id'])){
                    $_SESSION['Notifal_click_notif_id'][] = $notif_id;
                }
            }else{
                $_SESSION['Notifal_click_notif_id'][] = $notif_id;
            }
            wp_redirect($url);
            exit;
        }
    }




    public static function get_option_data($option_key, string $key='',$default_value=''){
        $value = get_option($option_key);
        if($value !== false){
            $data = json_decode($value);
            if(!empty($key)){
                return $data -> $key;
            }
            return $data;
        }
        return $default_value;
    }

    /**
     * save order meta
     * @param $order_id
     * @param $order
     * @return void
     */
    public function save_order_meta($order_id,$order){
        if (!empty($_SESSION['Notifal_click_notif_id']) && is_array($_SESSION['Notifal_click_notif_id'])) {
            $notif_ids = array_map('intval', $_SESSION['Notifal_click_notif_id']);
            $notif_ids = array_unique($notif_ids);
            sort($notif_ids);
            $notif_ids_json = json_encode($notif_ids);
            $order->add_meta_data('_notifal_notif_ids', $notif_ids_json);
            $order->save_meta_data();
            unset($_SESSION['Notifal_click_notif_id']);
        }
    }


    /**
     * Adds a custom metabox to the WooCommerce order edit page.
     *
     * @since 1.1.0
     */
    public function add_metabox_to_order() {
        $wc_custom_order=get_option('woocommerce_custom_orders_table_enabled');

        if($wc_custom_order==='no'){
            $screen = 'shop_order';
        }else{
            $screen = wc_get_page_screen_id( 'shop-order' );
        }
        add_meta_box(
            'notifal_ids_metabox',
            __('Related Notification IDs', Notifal_DOMAIN),
            [$this,'display_notif_data_in_metabox'],
            $screen,
            'side',
            'core'
        );
    }

    /**
     * Adds a custom metabox to the WooCommerce order edit page to display related notification IDs.
     *
     * @param WC_Order $order The order object.
     * @since 1.1.0
     */
    public function display_notif_data_in_metabox($post) {
        $wc_custom_order=get_option('woocommerce_custom_orders_table_enabled');
        if($wc_custom_order==='no'){
            if(!isset($post->ID)) {
                echo '<p>' . __('No related notification IDs found.', Notifal_DOMAIN) . '</p>';
                return;
            }
            $order = wc_get_order($post->ID);
            $notif_ids = json_decode($order->get_meta('_notifal_notif_ids'),true);
        }else{
            if(!isset($_GET['id'])) {
                echo '<p>' . __('No related notification IDs found.', Notifal_DOMAIN) . '</p>';
                return;
            }
            $order = wc_get_order($_GET['id']);
            $notif_ids = json_decode($order->get_meta('_notifal_notif_ids'),true);
        }
        if(!empty($notif_ids)){
            foreach ($notif_ids as $notif_id){
                $notif = notif::get_notif_by_id($notif_id);
                if(empty($notif)) return;
                $notif_edit_url = notif::notif_edit_url($notif_id);
                ?>
                <a target="_blank" href="<?php echo $notif_edit_url ?>"><?php echo $notif->title ?></a><br>
                <?php
            }
        }else{
            echo '<p>' . __('No related notification IDs found.', Notifal_DOMAIN) . '</p>';
        }


    }


    /**
     * Retrieves total revenue, total sold products, and total orders from Notifal notifications.
     *
     * @since 1.1.0
     *
     * @return array{
     *     total_revenue: string,
     *     total_sold_products: int,
     *     total_orders: int
     * } An array containing total revenue, total sold products, and total orders.
     */
    public static function get_total_revenue_data(): array {
        global $wpdb;

        $use_custom_orders_table = 'yes' === get_option('woocommerce_custom_orders_table_enabled', 'no');
        $orders_table = $use_custom_orders_table ? 'wc_orders' : 'posts';
        $meta_table = $use_custom_orders_table ? 'wc_orders_meta' : 'postmeta';
        $product_lookup_table = db::add_prefix_table('wc_order_product_lookup');

        $meta_key = '_notifal_notif_ids';
        $statuses = ['wc-processing', 'wc-completed'];

        $distinct_orders = db::get_distinct_rows(
            $meta_table,
            [$use_custom_orders_table ? 'order_id' : 'post_id'],
            ['meta_key' => $meta_key]
        );

        if (empty($distinct_orders)) {
            return [
                'total_revenue' => wc_price(0.0),
                'total_sold_products' => 0,
                'total_orders' => 0,
            ];
        }

        $order_ids = array_column($distinct_orders, $use_custom_orders_table ? 'order_id' : 'post_id');

        if (empty($order_ids)) {
            return [
                'total_revenue' => wc_price(0.0),
                'total_sold_products' => 0,
                'total_orders' => 0,
            ];
        }

        $order_placeholders = implode(',', array_fill(0, count($order_ids), '%d'));
        $status_placeholders = implode(',', array_fill(0, count($statuses), '%s'));

        if ($use_custom_orders_table) {
            $query_revenue = "
            SELECT o.id, o.total_amount
            FROM {$wpdb->prefix}{$orders_table} AS o
            WHERE o.id IN ({$order_placeholders})
            AND o.status IN ({$status_placeholders})
        ";
            $query_products = "
            SELECT plt.order_id, SUM(plt.product_qty) AS total_sold_products
            FROM {$product_lookup_table} AS plt
            INNER JOIN {$wpdb->prefix}{$orders_table} AS pm
            ON plt.order_id = pm.id
            WHERE plt.order_id IN ({$order_placeholders})
            AND pm.status IN ({$status_placeholders})
            GROUP BY plt.order_id
        ";
            $revenue_data = $wpdb->get_results($wpdb->prepare($query_revenue, ...$order_ids, ...$statuses), ARRAY_A);
            $product_data = $wpdb->get_results($wpdb->prepare($query_products, ...$order_ids,  ...$statuses), ARRAY_A);
            $total_revenue = 0.0;
            $total_sold_products = 0;
            $total_orders = count($revenue_data);

            foreach ($revenue_data as $order) {
                $total_revenue += (float) $order['total_amount'];
            }

            foreach ($product_data as $product) {
                $total_sold_products += (int) $product['total_sold_products'];
            }
        } else {
            $query_revenue = "
            SELECT p.ID, pm.meta_value AS total_amount
            FROM {$wpdb->prefix}{$orders_table} AS p
            INNER JOIN {$wpdb->prefix}{$meta_table} AS pm
            ON p.ID = pm.post_id
            WHERE pm.meta_key = '_order_total'
            AND p.ID IN ({$order_placeholders})
            AND p.post_status IN ({$status_placeholders})";
            $query_products = "
            SELECT plt.order_id, SUM(plt.product_qty) AS total_sold_products
            FROM {$product_lookup_table} AS plt
            INNER JOIN {$wpdb->prefix}{$orders_table} AS pm
            ON pm.id = plt.order_id
            WHERE plt.order_id IN ({$order_placeholders})
            AND pm.post_status IN ({$status_placeholders})
            GROUP BY plt.order_id";
            $revenue_data = $wpdb->get_results($wpdb->prepare($query_revenue, ...$order_ids, ...$statuses), ARRAY_A);
            $product_data = $wpdb->get_results($wpdb->prepare($query_products, ...$order_ids, ...$statuses), ARRAY_A);
            $total_revenue = 0.0;
            $total_sold_products = 0;
            $total_orders = count($revenue_data);

            foreach ($revenue_data as $order) {
                $total_revenue += (float) $order['total_amount'];
            }

            foreach ($product_data as $product) {
                $total_sold_products += (int) $product['total_sold_products'];
            }
        }

        return [
            'total_revenue' => wc_price($total_revenue),
            'total_sold_products' => $total_sold_products,
            'total_orders' => $total_orders,
        ];
    }











}