<?php
namespace Notifal;
use DirectoryIterator;
use mysqli_result;
use Notifal\notif\settings;

/**
 * notifications handler
 * @version 1.0.0
 */
if ( ! defined( 'ABSPATH' ) ) exit;
if (session_status() === PHP_SESSION_NONE) {
    session_start();
}
class notif
{
    use functions,translatejs;

    /**
     * Notifications database table name
     *
     * @since 1.0.0
     * @var string
     */
    public static string $notifal_table = 'notifal_notif';


    /**
     * Notifications meta table name
     *
     * @since 1.1.0
     * @var string
     */
    public static string $notifal_meta_table = 'notifal_notifmeta';

    /**
     * Backend asset path
     *
     * @since 1.0.0
     * @var string
     */
    public static string $notif_backend_asset_path = Notifal_URL . 'assets/backend/';

    /**
     * Sample data path
     *
     * @since 1.0.0
     * @var string
     */
    public static string $notif_data_path = Notifal_DIR . 'sample-data/';

    /**
     * Notif frontend asset path
     *
     * @since 1.0.0
     * @var string
     */
    public static string $notif_asset_path = Notifal_URL . 'assets/frontend/notif/';

    /**
     * Audio path
     *
     * @since 1.0.0
     * @var string
     */
    public static string $notif_audio_path = Notifal_URL . 'assets/audios/';


    /**
     * New notif id will be saved in this var
     *
     * @since 1.0.0
     * @var string
     */
    public static string $new_notif_id='';


    /**
     * Notif object
     *
     * @since 1.0.0
     * @var object
     */
    public static object $notif;


    /**
     * Notifs list, this will be used for filtering notifs in notifications list
     *
     * @since 1.0.0
     * @var array
     */
    public static array $notifs_list;


    /**
     * Total notifs count
     * @var string
     */
    public static string $total_notifs_list_counter;


    /**
     * Notif constructor
     */
    public function __construct()
    {
        $ajax_data=[
            'save_notif',
            'delete_notif',
            'notif_bulk_action',
            'update_notif_status',
        ];
        $nopriv_ajax_data = ['display_notif_actions','close_notif_actions'];
        $this->ajax_actions($this,$ajax_data);
        $this->noprivate_ajax_actions($this,$nopriv_ajax_data);
        add_action('admin_print_scripts', [$this, 'enqueue_setting_backend_script']);
        add_action('wp_enqueue_scripts', [$this, 'enqueue_front_script']);
        add_filter('admin_title', [$this,'change_notif_page_title']);
        add_action('admin_init',[$this,'admin_notif_init'],10,2);
        add_action('wp_footer',[$this,'add_notif_to_front']);
        add_action('admin_footer',[$this,'add_pro_popup']);

    }


    /**
     * Change {add new notif} page title to {Edit notif -notif id- notif-title} when
     * notif id found in url
     *
     * @since 1.0.0
     * @param $admin_title
     * @return mixed|string
     */
    public function change_notif_page_title($admin_title){
        if(isset($_GET['page'])){
            $page=$_GET['page'];
            if($page=='notif'){
                if(!self::is_add_notif_page()){
                    $notif_title = notif::get_notif_title();
                    return esc_html__('Edit Notif',Notifal_DOMAIN).' '.$_GET['notif_id'].': '.$notif_title;
                }
            }
            return $admin_title;

        }

        return $admin_title;
    }


    /**
     * get notif status
     * if in add new notif page return empty otherwise return current notif stauts
     *
     * @since 1.0.0
     * @return string
     */
    public static function get_notif_status(){
        if(notif::is_add_notif_page()){
            return '';
        }
        return self::$notif->status;
    }

    /**
     * Save notif via ajax
     *
     * @since 1.0.0
     * @return void
     */
    public function save_notif()
    {
        $notif_data = $_POST['notif_data'];
        $conditions = $notif_data['conditions'];
        $products = $notif_data['products'];
        $settings = $notif_data['settings'];
        $id = sanitize_text_field($notif_data['id']);
        $title = sanitize_text_field($notif_data['title']);
        $need_sanitize = [
                'settings'=>$settings
        ];
        $allowed_img = [
            'img' => array(
                'class' => true,
                'width' => true,
                'height' => true,
                'src' => true,
                'alt' => true,
                'decoding' => true,
                'srcset' => true,
                'sizes' => true,
            )
        ];
        $image_tags_keys=[
            'product_image'
        ];
        $textarea_keys=[
            'fake_orders_fn',
            'fake_orders_ln',
            'fake_orders_cities',
            'custom_css'
        ];
        $html_allowed_keys=[
            'custom_main_text',
            'custom_ordertime_text',
            'custom_stock_text',
            'action_button_text',
        ];
        $url_keys=[
            'custom_audio_src'
        ];
        foreach ($need_sanitize as $key=>$value){
            foreach ($value as $keyin=>$valuein){
                if(empty([$keyin])) return;
                if(in_array($keyin,$image_tags_keys)){
                    $value[$keyin]=wp_kses($valuein,$allowed_img);
                }else if(in_array($keyin,$textarea_keys)){
                    $value[$keyin]=sanitize_textarea_field($valuein);
                }else if(in_array($keyin,$html_allowed_keys)){
                    $value[$keyin]=wp_kses($valuein,wp_kses_allowed_html( 'post' ));
                }else if(in_array($keyin,$url_keys)){
                    $value[$keyin]=sanitize_url($valuein);
                }else if(strpos($keyin, 'color') !== false){
                    $value[$keyin]=sanitize_hex_color($valuein);
                }else{
                    $value[$keyin] = sanitize_text_field($valuein);
                }
            }
            $settings = $value;
        }
        $source = 'product';
        $type = json_encode($notif_data['type']);
        $status = $notif_data['status'];
        $user_id = get_current_user_id();
        $output_data=[];
        do_action('Notifal_before_notif_saved',$output_data,$status,$id,$products,$conditions,$settings);
        $encoded_conditions = json_encode(apply_filters('Notifal_notif_conditions_before_saved',$conditions));
        $encoded_products = json_encode(apply_filters('Notifal_notif_products_before_saved',$products));
        $encoded_settings = json_encode(apply_filters('Notifal_notif_settings_before_saved',$settings));
        if(empty($id)){
            $created_date = date('Y-m-d H:i:s');
            $insert_data = [
                'title' => $title,
                'source' => $source,
                'type' => $type,
                'conditions' => $encoded_conditions,
                'products' => $encoded_products,
                'settings' => $encoded_settings,
                'status' => $status,
                'created_date' => $created_date,
                'updated_date' => $created_date,
                'user_id' => $user_id,
            ];

            $insert_id=db::insert_in_table(self::$notifal_table,$insert_data);
            $output_data['edit_url'] = self::notif_edit_url($insert_id);
            if($insert_id){
                $output_data['message']='Inserted';
            }else{
                $output_data['message']='Failed';
            }
        }else{
            $updated_date = date('Y-m-d H:i:s');
            $update_data = [
                'title' => $title,
                'type' => $type,
                'conditions' => $encoded_conditions,
                'products' => $encoded_products,
                'settings' => $encoded_settings,
                'status' => $status,
                'updated_date' => $updated_date,
                'user_id' => $user_id,
            ];
            $update=self::update_notif($update_data,['id'=>$id]);
            $output_data['edit_url'] = self::notif_edit_url($id);
            if($update){
                $output_data['message']='Updated';
            }else{
                $output_data['message']='Failed';
            }
        }
        print_r(json_encode($output_data));

        unset($_SESSION['woo_real_order_notifs']);
        exit();
    }


    /**
     * Update notif, true if updated successfully otherwise return false
     *
     * @since 1.0.0
     * @param array $update_data
     * @param array $conditions
     * @param null $where_format
     * @return bool
     */
    public function update_notif(array $update_data,array $conditions,$where_format=null){
        $table_name = self::$notifal_table;
        $update=db::update_table($table_name,$update_data,$conditions,$where_format);
        return $update;
    }


    /**
     * Create notifs table
     *
     * @since 1.0.0
     * @return void
     */
    public static function create_notification_table(){
        $table_name= self::$notifal_table;
        $columns = [
          ['name'=>'id', 'type' => 'int(11)' , 'extra_data'=> 'NOT NULL AUTO_INCREMENT'],
          ['name'=>'title', 'type' => 'tinytext' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'source', 'type' => 'tinytext' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'type', 'type' => 'tinytext' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'conditions', 'type' => 'longtext' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'products', 'type' => 'longtext' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'settings', 'type' => 'longtext' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'status', 'type' => 'varchar(20)' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'created_date', 'type' => 'datetime' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'updated_date', 'type' => 'datetime' , 'extra_data'=> 'NOT NULL'],
          ['name'=>'user_id', 'type' => 'bigint(20)' , 'extra_data'=> 'NOT NULL'],
        ];
        $primary_key = 'id';
        db::create_table($table_name,$columns,$primary_key);
    }

    /**
     * Creates the `notifal_notifmeta` table for storing metadata related to notifications.
     *
     * Columns:
     * - `meta_id`: Primary key.
     * - `notif_id`: The ID of the notification.
     * - `meta_key`: The metadata key (e.g., 'total_views').
     * - `meta_value`: The metadata value (e.g., a JSON or string value).
     *
     * Indexes:
     * - `notif_id_idx`: Index for faster lookups by notification ID.
     * - `meta_key_idx`: Index for faster lookups by metadata key.
     *
     * @return void
     * @since 1.1.0
     */
    public static function create_notification_meta_table() {
        $table_name = self::$notifal_meta_table;
        $columns = [
            ['name' => 'meta_id', 'type' => 'bigint(20) unsigned', 'extra_data' => 'NOT NULL AUTO_INCREMENT'],
            ['name' => 'notif_id', 'type' => 'bigint(20) unsigned', 'extra_data' => 'NOT NULL'],
            ['name' => 'meta_key', 'type' => 'varchar(255)', 'extra_data' => 'NOT NULL'],
            ['name' => 'meta_value', 'type' => 'longtext', 'extra_data' => 'NULL'],
        ];
        $index_key = "KEY notif_id_idx (notif_id), KEY meta_key_idx (meta_key(191))";
        $primary_key = 'meta_id';
        db::create_table($table_name, $columns, $primary_key, $index_key);
    }

    /**
     * get template file names
     *
     * @since 1.0.0
     * @return array
     */
    public static function get_templates_files(){
        $notif_template_path = templates::$notif_template_path;
        $data=[];
        foreach (new DirectoryIterator($notif_template_path) as $file) {
            if ($file->isFile()) {
                $data['files_names'][] = $file->getFilename();
            }
        }
        if(maintenance::is_pro_active()){
            $notif_pro_template_path = \Notifal_Pro\templates::$notif_template_path;
            foreach (new DirectoryIterator($notif_pro_template_path) as $file) {
                if ($file->isFile()) {
                    $data['files_names'][] = $file->getFilename();
                }
            }
        }
        return $data;
    }

    /**
     * include template files
     *
     * @since 1.0.0
     * @return void
     */
    public static function add_template_preview(){
        $file_data=self::get_templates_files();
        $templates_files= $file_data['files_names'];
        sort($templates_files);
        $i=0;
        foreach ($templates_files as $file):
            $file_name_without_format=substr($file, 0, strpos($file, "."));
            ?>
            <div data-id="<?php echo $file ?>" class="template-preview <?php renderHtml::toggle_display(notif::get_notif_settings('selected_template'),$file_name_without_format,"equal",true , [$i=>!0]); ?>">
                <?php
                if(file_exists(templates::$notif_template_path.$file)){
                    include (templates::$notif_template_path.$file);
                }else if(file_exists(\Notifal_Pro\templates::$notif_template_path.$file)){
                    include (\Notifal_Pro\templates::$notif_template_path.$file);
                }
                ?>
            </div>
            <?php
            $i++;
        endforeach;
    }


    /**
     * Get notif object by id
     * if $id is not provided :
     * return current notif based on notif_id query value(on backend)
     * if notif_id is not provided return empty array
     * if notif not found redirected to notifs list
     * this function also set $notif var to use in this class if notif found
     *
     * if $id is provided return notif object
     *
     *
     * @since 1.0.0
     * @param string $id
     * @return array|object|\stdClass|void|null
     */
    public static function get_notif_by_id(string $id='')
    {
        if(empty($id)) {
            if(!isset($_GET['notif_id'])) return [];
            $id = $_GET['notif_id'];
            $notif = db::get_row_by_column(self::$notifal_table,'id',$id);
            if(empty($notif)){
                wp_redirect(menu_page_url('notifs-list'));
                exit();
            }else{
                self::$notif = db::get_row_by_column(self::$notifal_table,'id',$id);
                return self::$notif;
            }

        }else{
            return db::get_row_by_column(self::$notifal_table,'id',$id);
        }

    }


    /**
     * get current norif title on edit setting page otherwise return empty
     *
     * @since 1.0.0
     * @return string
     */
    public static function get_notif_title(){
        if(self::is_add_notif_page()){
            return '';
        }
        return stripslashes(self::$notif->title);
    }

    /**
     * get current norif id on edit setting page otherwise return empty
     *
     * @since 1.0.0
     * @return string
     */
   public static function get_notif_id(){
        if(self::is_add_notif_page()){
            return '';
        }
        return self::$notif->id;
    }


    /**
     * if $notif is empty return custom css string otherwise add position styles
     * to custom css then return it
     *
     * @since 1.0.0
     * @param $notif
     * @return string
     */
    public static function set_notif_template_styles($notif='') {
        $styles=notif::get_notif_settings('custom_css',false,$notif);
        if(!empty($notif)){
            $distance_top=notif::get_notif_settings('distance_top',false,$notif);
            $distance_bottom=notif::get_notif_settings('distance_bottom',false,$notif);
            $distance_left=notif::get_notif_settings('distance_left',false,$notif);
            $distance_right=notif::get_notif_settings('distance_right',false,$notif);
            $styles.='#notifal-template-'.$notif->id .'{';
            $position=notif::get_notif_settings('position',false,$notif);
            $position_array = explode('-',$position);
            if($position_array[0]=='top'){
                $styles .= 'top:' . $distance_top .'px;';
            }else{
                $styles .= 'bottom:' . $distance_bottom .'px;';
            }
            switch ($position_array[1]){
                case 'center':{
                    $styles .= 'right:50%; transform: translate(50%,0);';
                    break;
                }
                case 'left':{
                    $styles .= 'left:' . $distance_left .'px;';
                    break;
                }
                case 'right':{
                    $styles .= 'right:' . $distance_right .'px;';
                    break;
                }
            }
            $styles .= '}';
        }

        return $styles;
    }

    /**
     * Update notif status via ajax
     *
     * @since 1.0.0
     * @return void
     */
    public function update_notif_status(){
        $id=$_POST['id'];
        $status=$_POST['status'];
        if($status==='true'){
            $stauts_text = 'Active';
        }else{
            $stauts_text = 'Inactive';
        }
        $update_data=['status'=>$stauts_text];
        $update_data = apply_filters('before_update_status',$update_data,$id);
        if( $this->update_notif($update_data,['id'=>$id])){
            echo 'updated';
        }else{
            echo 'error';
        }
        unset($_SESSION['woo_real_order_notifs']);
        exit();
    }


    /**
     * Filter notifs list data by query data
     *
     * @since 1.0.0
     * @return void
     */
    public static function get_notif_lists_data()
    {
        $table_name= self::$notifal_table;
        $conditions = [];
        $offset = 0;
        $dataKeys='id,title,created_date,conditions,products,status';
        if(isset($_GET['status'])){
            $status = $_GET['status'];
            $conditions[]="status='$status'";
        }
        if(isset($_GET['search'])){
            $search = $_GET['search'];
            $conditions[]="title LIKE '%$search%'";
        }
        if(isset($_GET['pageNum'])){
            $pageNum = $_GET['pageNum'];
            $offset = ($pageNum-1) *10;
        }
        if(!empty($conditions)){
            self::$notifs_list = db::get_table_rows($table_name,$dataKeys,$conditions,'id',10,$offset);
            self::$total_notifs_list_counter = sizeof(db::get_table_rows($table_name,$dataKeys,$conditions,'id',-1));
        }
        else{
            self::$notifs_list = db::get_table_rows($table_name,$dataKeys,[],'id',10,$offset);
            self::$total_notifs_list_counter = sizeof(db::get_table_rows($table_name,$dataKeys,[],'id',-1));
        }

    }


    /**
     * Return notifs by status, if $status is empty return all status
     *
     * @since 1.0.0
     * @param $status
     * @return array|object|\stdClass[]|null
     */
    public static function get_notifs_by_status($status=''){
        $table_name= self::$notifal_table;
        $conditions = ["status='$status'"];
        if(empty($status)){
            return db::get_table_rows($table_name,'all',[],'id',-1);
        }
        return db::get_table_rows($table_name,'all',$conditions,'id',-1);
    }

    /**
     * return notif filter link
     *
     * @since 1.0.0
     * @param $args
     * @return string
     */
    public static function notifs_list_filter_link($args=[]){
        $admin_url=admin_url('admin.php');
        $filters = array(
            'page' => 'notifs-list',
        );
        foreach ($args as $key=>$value){
            $filters[$key]= $value;
        }
        $notif_list_filter_url=add_query_arg($filters,$admin_url);
        return $notif_list_filter_url;
    }

    /**
     * return notif product data
     * if $notif_obj is empty get notif by $id on backend
     * if $data is provided return key on $data otherwise return all
     *
     * @since 1.0.0
     * @param string $data
     * @param string $id
     * @param mixed $notif_obj
     * @return array|mixed|string
     * @since 1.0.0
     */
    public static function get_notif_products(string $data='type', string $id='',  $notif_obj=''){
        if(empty($notif_obj)){
            if(empty($id)){
                if(self::is_add_notif_page() || empty(self::$notif->products)){
                    return '';
                }
                $products = self::$notif->products;
            }
            else{
                $notif=self::get_notif_by_id($id);
                $products = $notif->products;
            }
        }else{
            $products = $notif_obj->products;
        }

        $products_all_data=json_decode($products,false);
        $products_data=[
            'type'=>$products_all_data->notif_products ?? '',
            'selected_products'=>$products_all_data->selected_products ?? '',
            'selected_categories'=>$products_all_data->selected_categories ?? '',
        ];
        if(!empty($data)){
            return $products_data[$data];
        }else{
            return $products_data;
        }


    }

    /**
     * get notif page conditions
     * if $data is not provided return all otherwise return key of condition
     * if $id is empty get current notid condition
     *
     * @since 1.0.0
     * @param string $data
     * @param string $id
     * @return mixed|string
     */
    public static function get_notif_conditions(string $data='type', string $id=''){
        if(empty($id)){
            if(self::is_add_notif_page() || empty(self::$notif->conditions)){
                return '';
            }
            $conditions= self::$notif->conditions;

        }else{
            $notif=self::get_notif_by_id($id);
            $conditions = $notif->conditions;
        }
        $conditions_all_data=json_decode($conditions,false);
        $conditions_data=[
            'type'=>$conditions_all_data->page_type ?? '',
            'selected_pages'=>$conditions_all_data->selected_pages ?? '',
            'selected_category_pages'=>$conditions_all_data->selected_category_pages ?? '',
            ];

        return $conditions_data[$data];


    }

    /**
     * Get products id by notif
     * if pro is installed this function works otherwise return empty
     *
     * @since 1.0.0
     * @param object $notif
     * @return mixed|null
     */
    public static function get_product_ids_by_notif(object $notif){
        $product_ids=[];
        return apply_filters('get_products_ids_by_notif',$product_ids,$notif);
    }


    /**
     * Get notif settings
     *
     * if $htmlText is true return html content otherwise return plain text
     * if $notif is not provided return current notif on backed
     * if is on add notif page return empty
     *
     *
     * @since 1.0.0
     * @param string $key
     * @param bool $htmlText
     * @param mixed $notif
     * @return string
     */
    public static function get_notif_settings(string $key, bool $htmlText=false, $notif=''){
        if(self::is_add_notif_page()){
            return '';
        }
        if(empty($notif)){
            $settings = json_decode(self::$notif->settings,false);
        }else{
            $settings = json_decode($notif->settings,false);
        }
        if($htmlText){
            return htmlspecialchars(stripslashes($settings->$key));
        }
        return $settings->$key;
    }


    /**
     * return true is on setting page otherwise return false
     *
     * @since 1.0.0
     * @return bool
     */
    public static function is_notif_setting_page(): bool
    {
        if(isset($_GET['page'])){
            if($_GET['page']=='notif'){
                return true;
            }
        }
        return false;
    }

    /**
     * Get active notifs of a page
     * if $pageid is empty get current page id, if is category page get current category page id.
     *
     * @param string $pageid
     * @return array|object|\stdClass[]|null
     * @since 1.0.0
     */
    public static function get_active_notifs_by_page(string $pageid=''){
        $conditions = ['(JSON_CONTAINS(conditions, \'"all-pages"\',"$.page_type") AND status="Active")'];

        if(empty($pageid)){
            if(maintenance::is_woocommerce_active()){
                if(is_product_category()){
                    global $wp_query;
                    $cat_id=$wp_query->get_queried_object()->term_id;
                    $pageid = $cat_id;
                    $conditions = apply_filters('Notifal_category_conditions_query',$conditions,$pageid);

                }else{
                    if(is_shop()){
                        $pageid = get_option( 'woocommerce_shop_page_id' );
                    }else{
                        $pageid = get_the_ID();
                    }
                    $conditions = apply_filters('Notifal_page_conditions_query',$conditions,$pageid);
                }
            }else{
                $pageid = get_the_ID();
                $conditions = apply_filters('Notifal_page_conditions_query',$conditions,$pageid);
            }


        }

        if(!maintenance::is_pro_active()){
            $limit=1;
        }else{
            $limit=-1;
        }

        $notifs= db::get_table_rows(self::$notifal_table,'all',$conditions,'id',$limit,0,'OR');
        return $notifs;
    }

    /**
     * notif bulk action via ajax
     *
     * @since 1.0.0
     * @return void
     */
    public function notif_bulk_action(){
        $bulk_action=$_POST['bulk_action'];
        $selected_notifs=$_POST['selected_notifs'];
        $sanitized_selected_notifs = [];
        foreach ($selected_notifs as $notif_id){
            $sanitized_selected_notifs[]=sanitize_text_field($notif_id);
        }
        $error = false;
        if($bulk_action=='delete'){
            foreach ($sanitized_selected_notifs as $notif_id) {
                if (!self::delete_notif($notif_id)) {
                    $error = true;
                }
            }
            if($error){
                echo 'Failed to delete';
            }else{
                echo 'deleted';
            }
        }else if($bulk_action=='activate'){
            $update_data = [
                'status' => 'Active',
            ];
            foreach ($sanitized_selected_notifs as $notif_id) {
                if (!self::update_notif($update_data,['id'=>$notif_id])) {
                    $error = true;
                }
            }
            if($error){
                echo 'Failed to activate';
            }else{
                echo 'activated';
            }
        }else if($bulk_action=='deactivate'){
            $update_data = [
                'status' => 'Inactive',
            ];
            foreach ($sanitized_selected_notifs as $notif_id) {
                if (!self::update_notif($update_data,['id'=>$notif_id])) {
                    $error = true;
                }
            }
            if($error){
                echo 'Faild to deactivate';
            }else{
                echo 'deactivated';
            }
        }
        unset($_SESSION['woo_real_order_notifs']);
        exit();

    }

    /**
     * delete notif
     *
     * @param $id
     * @return bool|int|mysqli_result|string
     * @since 1.0.0
     */
    private function delete_notif($id){
        unset($_SESSION['woo_real_order_notifs']);
        $id = sanitize_text_field($id);

        return db::remove_row(self::$notifal_table,['id'=>$id]);

    }


    /**
     * return true if is add notif page otherwise return false
     *
     * @since 1.0.0
     * @return bool
     */
    public static function is_add_notif_page(){
        if(isset($_GET['page'])){
            if($_GET['page']=='notif' && !isset($_GET['notif_id'])){
                return true;
            }
        }
        return false;
    }


    /**
     * return true if is notif list page otherwise return false
     * @return bool
     */
    public static function is_notif_list_page(){
        if(isset($_GET['page'])){
            if($_GET['page']=='notifs-list'){
                return true;
            }
        }
        return false;
    }


    /**
     * return true if is admin dashboard page
     *
     * @since 1.1.0
     * @return bool
     */
    public static function is_dashboard_page(): bool
    {
        if(isset($_GET['page'])){
            if($_GET['page']=='Notifal-dashboard'){
                return true;
            }
        }
        return false;
    }

    /**
     * Insert pre created Notifs
     *
     * @since 1.0.0
     * @return void
     */
    public static function insert_precreated_notifs() {
        global $wpdb;

        $table_name = $wpdb->prefix.self::$notifal_table;

        $row = $wpdb->get_var("SELECT COUNT(*) FROM {$table_name}");
        if ($row > 0) {
            return;
        }
        $sql_file = self::$notif_data_path . 'pre-created-notifs.sql';
        if (file_exists($sql_file)) {
            $sql_content = file_get_contents($sql_file);
            $sql_content=str_replace('wp_notifal', $table_name, $sql_content);
            $wpdb->query($sql_content);
        }
    }


    /**
     * return true if is Notifal pages
     *
     * @since 1.0.0
     * @return bool
     */
    public static function is_notifal_pages(){
        if(isset($_GET['page'])){
            if(in_array($_GET['page'],['notifs-list','notif','Notifal-dashboard'])){
                return true;
            }
        }
        return false;
    }


    /**
     * set notif color by its key setting, if is add notif page return $default
     * @param $setting_color_key
     * @param $default
     * @param $notif
     * @return void
     * @since 1.0.0
     */
    public static function set_notif_color($setting_color_key,$default,$notif='') {
        if(notif::is_add_notif_page()){
            echo $default;
        }
        else{
            if(is_admin()){
                echo notif::get_notif_settings($setting_color_key);
            }else{
                echo notif::get_notif_settings($setting_color_key,false,$notif);
            }
        }
    }

    /**
     * return notif edit url
     *
     * @since 1.0.0
     * @param $id
     * @return string
     */
    public static function notif_edit_url($id){
        $admin_url=admin_url('admin.php');
        $notif_edit_url=add_query_arg(array(
            'page' => 'notif',
            'notif_id'=> $id,
        ),$admin_url);
        return $notif_edit_url;
    }


    /**
     * Enqueue setting scripts and localized data
     *
     * @since 1.0.0
     * @return void
     */
    public function enqueue_setting_backend_script(): void
    {
        if(self::is_notif_setting_page()){
            $ajax_objects = ['save_notif'];
            $data = [
                'order_template_path'=>templates::$notif_template_path,
                'audio_path'=>self::$notif_audio_path,
                'default_templates_colors' => json_encode(settings::get_color_defaults_by_template()),
                'order_tags' => json_encode(settings::require_orders_tag()),
                'stock_tags' => json_encode(settings::stock_tags()),
                'discount_tags' => json_encode(settings::discount_tags()),
                'is_pro_active' => maintenance::is_pro_active(),
                'is_woocommerce_active' => maintenance::is_woocommerce_active()
            ];
            wp_enqueue_media();
            wp_enqueue_style('AddNotif', self::$notif_backend_asset_path . 'add-new-notif.min.css', '', Notifal_VERSION);
            wp_enqueue_script('AddNotif', self::$notif_backend_asset_path . 'add-new-notif.min.js', ['jquery'], Notifal_VERSION);
            $this->enqueue_translation_data('AddNotif','translated_strings',$this->backend_notif_translations());
            $this->localize_data_scripts('AddNotif','add_notif_data',$data);
            $this->localize_ajax_scripts('AddNotif',$ajax_objects);
        }else if(self::is_notif_list_page()){
            $ajax_objects = ['notif_bulk_action','update_notif_status'];
            $data = [
                'is_pro_active' => maintenance::is_pro_active(),
                'is_woocommerce_active' => maintenance::is_woocommerce_active(),
            ];
            wp_enqueue_style('notifalList', self::$notif_backend_asset_path . 'notifs-list.min.css', '', Notifal_VERSION);
            wp_enqueue_script('notifalList', self::$notif_backend_asset_path . 'notifs-list.min.js', ['jquery'], Notifal_VERSION);
            $this->enqueue_translation_data('notifalList','translated_strings',$this->backend_notif_translations());
            $this->localize_ajax_scripts('notifalList',$ajax_objects);
            $this->localize_data_scripts('notifalList','notif_list_data',$data);

        }else if(self::is_dashboard_page()){
            wp_enqueue_style('notifalDashboard', self::$notif_backend_asset_path . 'dashboard.css', '', Notifal_VERSION);

            wp_enqueue_script('notifalDashboard', self::$notif_backend_asset_path . 'dashboard.js', ['jquery'], Notifal_VERSION);

        }

    }


    /**
     * Notif init on admin
     *
     * @since 1.0.0
     * @return void
     */
    public function admin_notif_init(){

        if(!notif::is_add_notif_page() && !notif::is_notif_list_page()){
            self::get_notif_by_id();
        }else if(notif::is_add_notif_page()){
            self::$new_notif_id = db::get_latest_id(self::$notifal_table);

        }else if(notif::is_notif_list_page()){
            self::get_notif_lists_data();

        }


    }

    /**
     * render active notif data to prepare it for js on frontend
     *
     * @since 1.0.0
     * @return array|string
     */
    public function render_notif_data(){
        $notifs = self::get_active_notifs_by_page();
        if(empty($notifs)) return '';

        $data = [];
        foreach ($notifs as $notif){
            $notif_id = $notif->id;
            $index = 'notif_data_'.$notif_id;
            $products_data = notif::get_notif_products('','',$notif);
            $data[$index]['notif_counter']= self::get_notif_settings('notif_counter',false,$notif);
            $data[$index]['position']= self::get_notif_settings('position',false,$notif);
            $data[$index]['notif_counter_type']= self::get_notif_settings('notif_counter_type',false,$notif);
            $data[$index]['showing_transition_speed']= self::get_notif_settings('showing_transition_speed',false,$notif);
            $data[$index]['hiding_transition_speed']= self::get_notif_settings('hiding_transition_speed',false,$notif);
            $data[$index]['show_transition_style']= self::get_notif_settings('show_transition_style',false,$notif);
            $data[$index]['if_close_notif']= self::get_notif_settings('if_close_notif',false,$notif);
            $data[$index]['hide_transition_style']= self::get_notif_settings('hide_transition_style',false,$notif);
            $data[$index]['display_duration']= self::get_notif_settings('display_duration',false,$notif);
            $data[$index]['initial_delay']= self::get_notif_settings('initial_delay',false,$notif);
            $data[$index]['interval_delay']= self::get_notif_settings('interval_delay',false,$notif);
            $data[$index]['audio_volume']= self::get_notif_settings('audio_volume',false,$notif);
            $data[$index]['mobile_inactive']= self::get_notif_settings('mobile_inactive',false,$notif);
            $order_threshold_num = notif::get_notif_settings('woo_orders_time',false,$notif);

            if(maintenance::is_woocommerce_active()){
                $data[$index]['display_duplicate_orders']= self::get_notif_settings('display_duplicate_orders',false,$notif);
                $data[$index]['stock_max_quantity']= self::get_notif_settings('stock_max_quantity',false,$notif);
                $data[$index]['products_data_type']= $products_data['type'];
                $products_ids = self::get_product_ids_by_notif($notif);

            }
            $notif_main_text = notif::get_notif_settings('custom_main_text',false,$notif);
            $first_additional_info_text = notif::get_notif_settings('first_additional_info_text',false,$notif);
            $second_additional_info_text = notif::get_notif_settings('second_additional_info_text',false,$notif);
            $action_button_text = notif::get_notif_settings('action_button_text',false,$notif);
            $first_additional_info = $data[$index]['first_additional_info']=notif::get_notif_settings('first_additional_info',false,$notif);
            $second_additional_info = $data[$index]['second_additional_info']=notif::get_notif_settings('second_additional_info',false,$notif);
            $action_button = $data[$index]['action_button']=notif::get_notif_settings('action_button',false,$notif);
            $all_tags = settings::get_all_tags();
            $discount_tags = settings::discount_tags();
            $text_status[]=[$second_additional_info=>$second_additional_info_text];
            $text_status[]=[$action_button=>$action_button_text];
            $text_status[]=[$first_additional_info=>$first_additional_info_text];
            $custom_texts = [$notif_main_text];
            foreach ($text_status as $text_condition){
                foreach ($text_condition as $key=>$value){
                    if($key=='true'){
                        $custom_texts[]= $value;
                    }
                }

            }

            if(maintenance::is_woocommerce_active()){
                $woo_orders_active =  $data[$index]['woo_orders_active'] = notif::get_notif_settings('woo_orders_active',false,$notif);
                $fake_orders_active = $data[$index]['f_orders_active'] = notif::get_notif_settings('fake_orders_active',false,$notif);
                $require_order_tags = settings::require_orders_tag();
                $need_order_data = false;
                $need_replace = false;
                $need_sale_filter = false;
                $need_sale_end_date = false;
                foreach ($require_order_tags as $tag){
                    foreach ($custom_texts as $notif_text) {
                        if (strpos($notif_text, $tag) !== false) {
                            $need_order_data = true;
                            break;
                        }
                    }
                }
                foreach ($require_order_tags as $tag){
                    foreach ($custom_texts as $notif_text) {
                        if (strpos($notif_text, $tag) !== false) {
                            $need_order_data = true;
                            break;
                        }
                    }
                }
                foreach ($custom_texts as $notif_text) {
                    if (strpos($notif_text, '{discount_ends_in}') !== false) {
                        $need_sale_end_date = true;
                        break;
                    }
                }
                foreach ($all_tags as $tag){
                    foreach ($custom_texts as $notif_text) {
                        if (strpos($notif_text, $tag) !== false) {
                            $need_replace = true;
                            break;
                        }
                    }
                }

                foreach ($discount_tags as $tag){
                    foreach ($custom_texts as $notif_text) {
                        if (strpos($notif_text, $tag) !== false) {
                            $need_sale_filter = true;
                            break;
                        }
                    }
                }
                $data[$index]['need_replace'] = $need_replace;
                $data[$index]['need_sale_filter'] = $need_sale_filter;
                $data[$index]['need_order_data'] = $need_order_data;
                if($need_order_data){
                    if($woo_orders_active=='true'){
                        if(empty($_SESSION['woo_real_order_notifs'])){
                            $orders=plugin::get_instance()->woocommerce->get_orders_by_notif($notif);
                            $_SESSION['woo_real_order_notifs'] = $orders;
                        }else{
                            $orders =$_SESSION['woo_real_order_notifs'];
                        }

                        $i=0;
                        if(!empty($orders)){
                            foreach($orders as $order){
                                $order_index = "order_$i";
                                $data[$index]['real_order_data'][$order_index]['order_id'] = $order['order_id'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['order_product_ids'] = $order['order_product_ids'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['first_name'] = $order['first_name'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['last_name'] = $order['last_name'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['order_city'] = $order['order_city'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['order_country'] = $order['order_country'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['order_state'] = $order['order_state'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['order_time'] = $order['order_time'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['product_name'] = $order['product_name'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['product_id'] = $order['product_id'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['product_image'] = $order['product_image'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['product_stock'] = $order['product_stock'] ?? '';
                                $data[$index]['real_order_data'][$order_index]['product_link'] = $order['product_link'];
                                $i++;
                            }
                        }

                    }

                    if($fake_orders_active=='true'){
                        $fake_order_day=self::get_notif_settings('fake_orders_time',false,$notif);
                        $data[$index]['f_orders_time'] = $this->generate_random_times($data[$index]['notif_counter'],$fake_order_day);
                        $fake_orders_fn=self::get_notif_settings('fake_orders_fn',false,$notif);
                        $fake_orders_ln=self::get_notif_settings('fake_orders_ln',false,$notif);
                        $fake_orders_cities=self::get_notif_settings('fake_orders_cities',false,$notif);
                        $fake_texts = ['f_orders_fn'=>$fake_orders_fn,'f_orders_ln'=>$fake_orders_ln,'f_orders_cities'=>$fake_orders_cities];
                        foreach ($fake_texts as $key=>$value){

                            $data[$index][$key] = array_filter(preg_split("/\r\n|\n|\r/", $value));
                        }

                        if(in_array($products_data['type'],['all-products-except','all-products'])){
                            $fake_product_data = plugin::get_instance()->woocommerce->get_random_product($data[$index]['notif_counter'],$products_ids,$need_sale_filter,$need_sale_end_date);
                            $j=0;
                            foreach ($fake_product_data as $product){
                                $data[$index]['f_order_data']['f_product_data_'.$j]['id']= $product['id'];
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_name']= $product['name'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_image']= $product['image'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_stock']= $product['stock'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_regular_price']= $product['product_regular_price'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_sale_price']= $product['product_sale_price'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_link']= $product['product_link'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_discount_amount']= $product['product_discount_amount'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_discount_percent']= $product['product_discount_percent'] ?? '';
                                $data[$index]['f_order_data']['f_product_data_'.$j]['discount_ends_in']= $product['discount_ends_in'] ?? '';
                                $j++;
                            }
                        }else{
                            $j=0;
                            foreach($products_ids as $product_id){
                                $product = wc_get_product($product_id);
                                if($product->is_type( 'variable' )){
                                    $variation_ids = $product->get_children();
                                    $variation_id = array_rand($variation_ids);
                                    $product = wc_get_product($variation_ids[$variation_id]);
                                }
                                $data[$index]['f_order_data']['f_product_data_'.$j]['id']= $product->get_id();
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_name']= $product->get_name();
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_image']= $product->get_image([80,80]);
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_stock']= $product->get_stock_quantity();
                                $product_regular_price = $product -> get_regular_price();
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_regular_price'] = wc_price($product_regular_price);
                                if(!empty($product -> get_sale_price())){
                                    $product_sales_price = $product -> get_regular_price();
                                    $data[$index]['f_order_data']['f_product_data_'.$j]['product_sale_price'] = wc_price($product_sales_price);
                                }
                                $data[$index]['f_order_data']['f_product_data_'.$j]['product_link'] = $product -> get_permalink();
                                if(!empty($product -> get_date_on_sale_to())){
                                    $data[$index]['f_order_data']['f_product_data_'.$j]['discount_ends_in'] = $this->time_substract($product -> get_date_on_sale_to()->date('Y-m-d H:i:s'),true);
                                }
                                if(!empty($product -> get_sale_price())){
                                    $discount_amount = $product_regular_price - $product_sales_price;
                                    $discount_percent = floor(($discount_amount / $product_regular_price) * 100).'%' ;
                                    $data[$index]['f_order_data']['f_product_data_'.$j]['product_discount_amount'] = wc_price($discount_amount);
                                    $data[$index]['f_order_data']['f_product_data_'.$j]['product_discount_percent'] = (int)$discount_percent. '%';

                                }
                                $j++;
                            }
                        }

                    }


                }

                else{
                    if(empty($products_ids) || $products_data['type']=='all-products-except'){

                        $product_data = plugin::get_instance()->woocommerce->get_random_product($data[$index]['notif_counter'],$products_ids,$need_sale_filter,$need_sale_end_date);
                        $data[$index]['product_data'] = $product_data;
                        $data[$index]['need_sale_filter'] = $need_sale_filter;
                        $data[$index]['need_sale_end_date'] = $need_sale_end_date;
                        $j=0;
                        foreach ($product_data as $product){
                            $data[$index]['product_data']['data_'.$j]['id']= $product['id'];
                            $data[$index]['product_data']['data_'.$j]['product_name']= $product['name'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['product_image']= $product['image'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['product_stock']= $product['stock'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['product_regular_price']= $product['product_regular_price'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['product_sale_price']= $product['product_sale_price'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['product_link']= $product['product_link'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['product_discount_amount']= $product['product_discount_amount'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['product_discount_percent']= $product['product_discount_percent'] ?? '';
                            $data[$index]['product_data']['data_'.$j]['discount_ends_in']= $product['discount_ends_in'] ?? '';
                            $j++;
                        }
                    }else{
                        $j=0;
                        $data[$index]['products_id']= $products_ids;
                        foreach($products_ids as $product_id){
                            $product = wc_get_product($product_id);
                            if($product->is_type( 'variable' )){
                                $variation_ids = $product->get_children();
                                $variation_id = array_rand($variation_ids);
                                $product = wc_get_product($variation_ids[$variation_id]);
                            }
                            $data[$index]['product_data']['data_'.$j]['id']= $product->get_id();
                            $data[$index]['product_data']['data_'.$j]['product_name']= $product->get_name();
                            $data[$index]['product_data']['data_'.$j]['product_stock']= $product->get_stock_quantity();
                            $data[$index]['product_data']['data_'.$j]['product_image']= $product->get_image([80,80]);
                            $product_regular_price = $product -> get_regular_price();
                            $data[$index]['product_data']['data_'.$j]['product_regular_price']=wc_price($product_regular_price);
                            if(!empty($product -> get_sale_price())){
                                $product_sales_price = $product -> get_sale_price();
                                $data[$index]['product_data']['data_'.$j]['product_sale_price']=wc_price($product_sales_price);
                            }
                            $data[$index]['product_data']['data_'.$j]['product_link'] = $product -> get_permalink();
                            if(!empty($product -> get_date_on_sale_to())){
                                $data[$index]['product_data']['data_'.$j]['discount_ends_in'] = $this->time_substract($product->get_date_on_sale_to()->date('Y-m-d H:i:s'),true);
                            }
                            if(!empty($product -> get_sale_price())){
                                $discount_amount = $product_regular_price - $product_sales_price;
                                $discount_percent = floor(($discount_amount / $product_regular_price) * 100).'%';
                                $data[$index]['product_data']['data_'.$j]['product_discount_amount'] = $discount_amount;
                                $data[$index]['product_data']['data_'.$j]['product_discount_percent'] = $discount_percent;
                            }
                            $j++;
                        }
                    }
                }
            }


            $data[$index]=apply_filters('Notifal_after_render_notif_data' , $data[$index],$notif,$order_threshold_num);
        }

//        return apply_filters('Notifal_after_render_notifs_data',$data,$notifs);
        return $data;

    }


    /**
     * Enqueue frontend scripts and data
     *
     * @since 1.0.0
     * @return void
     */
    public function enqueue_front_script(){
        wp_enqueue_script('notifal', self::$notif_asset_path . 'scripts.min.js', ['jquery'], Notifal_VERSION);
        wp_enqueue_style('notifal', self::$notif_asset_path . 'styles.min.css', [], Notifal_VERSION);
        $notif_data = $this->render_notif_data();
        $encoded_notif_data=json_encode($notif_data);
        $ajax_objects = ['display_notif_actions','close_notif_actions'];

        $this->localize_data_scripts('notifal','notifal',[
            'active_notifs_data'=>$encoded_notif_data,
            'all_notif_tags' => json_encode(settings::get_all_tags()),
            'is_pro_active' => maintenance::is_pro_active(),
            'is_woocommerce_active' => maintenance::is_woocommerce_active(),
            'pro_tags' => json_encode(settings::pro_tags()),
            'page_id' => get_the_ID(),
        ]);
        $this->enqueue_translation_data('notifal','translated_strings',$this->frontend_notif_translations());
        $this->localize_ajax_scripts('notifal',$ajax_objects);

        wp_enqueue_style('dashicons');
    }


    /**
     * Add active notif templates to frontend
     *
     * @since 1.0.0
     * @return void
     */
    public function add_notif_to_front(){
        if(is_admin()) return;
        $notifs = self::get_active_notifs_by_page();
        if(!empty($notifs)):
            foreach ($notifs as $notif):
                $template = self::get_notif_settings('selected_template',false,$notif);
                $audio = self::get_notif_settings('sound_active',false,$notif);
                $is_image_allowed = self::get_notif_settings('product_image',false,$notif);
                $template_id = 'notifal-template-'.$notif->id;
            ?>
            <div class="notifal-template-parent" id="<?php echo $template_id ?>">
                <div class="notifal-template notifal-d-none" data-id="<?php echo 'notif_data_'.$notif->id; ?>" >
                    <?php
                    if($audio === 'true'):
                        $audio_name =  self::get_notif_settings('sound_audio',false,$notif);
                        if($audio_name==='custom-audio'){
                            $src = self::get_notif_settings('custom_audio_src',false,$notif);
                        }else{
                            $src = notif::$notif_audio_path .$audio_name.'.mp3';
                        }
                        ?>
                        <audio class="notifal-audio" src="<?php echo $src  ?>">
                        </audio>
                    <?php endif;
                    if(file_exists(templates::$notif_template_path.$template.'.php')) {
                        include(templates::$notif_template_path . $template . '.php');
                    }else if(file_exists(\Notifal_Pro\templates::$notif_template_path . $template . '.php')){
                        include(\Notifal_Pro\templates::$notif_template_path . $template . '.php');
                    }
                    ?>
                </div>

            </div>
                <?php
            endforeach;
        endif;
    }


    /**
     * Add pro popup on backend
     *
     * @since 1.0.0
     * @return void
     */
    public function add_pro_popup(){
        if(!is_admin() || !self::is_notifal_pages() || maintenance::is_pro_active()) return;
        $function = $this;
        include (templates::get_backend_template_path('pro-popup.php'));

    }


    /**
     * Ajax function is executed when notif is shown
     *
     * @since 1.1.0
     *
     * @return void
     */
    public function display_notif_actions(){
        $notif_id = (int)$_POST['notif_id'];
        $page_id = (int)$_POST['page_id'];
        $counter = (int)$_POST['counter'];

        do_action('trigger_display_notif_actions',$notif_id,$page_id,$counter);

        exit();
    }

    /**
     * Ajax function is executed when notif is closed
     *
     * @since 1.1.0
     *
     * @return void
     */
    public function close_notif_actions(){
        $notif_id = (int)$_POST['notif_id'];
        $page_id = (int)$_POST['page_id'];
        $counter = (int)$_POST['counter'];

        do_action('trigger_close_notif_actions',$notif_id,$page_id,$counter);

        exit();
    }












}
