<?php
namespace Notifal\notif;
use Notifal\maintenance;


/***
 * Notif Settings
 * notif settings handler
 */

class settings{

    /**
     * Return notif settings main menu
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function notif_setting_main_menu(): array
    {
        $menu=[
            ['name'=>esc_html__('Generals',Notifal_DOMAIN),'data-action'=>'notifal-setting-generals'],
            ['name'=>esc_html__('Visuals',Notifal_DOMAIN),'data-action'=>'notifal-setting-visuals'],
        ];
        if(maintenance::is_woocommerce_active()){
            $menu[]=['name'=>esc_html__('Source',Notifal_DOMAIN),'data-action'=>'notifal-setting-source'];
        }
        $menu[]=['name'=>esc_html__('Display Conditions',Notifal_DOMAIN),'data-action'=>'notifal-setting-conditions'];
        return $menu;
    }

    /**
     * Return notif target products options
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function notif_products_condition(): array
    {
        $products_conditions=[
            ['name'=>esc_html__('All Products',Notifal_DOMAIN),'value'=>'all-products'],
            ['name'=>esc_html__('All Except Selected',Notifal_DOMAIN),'value'=>'all-products-except','Pro'=>true],
            ['name'=>esc_html__('Selected Products Only',Notifal_DOMAIN),'value'=>'specific-products','Pro'=>true],
            ['name'=>esc_html__('No Product',Notifal_DOMAIN),'value'=>'no-products'],
        ];
        return $products_conditions;
    }

    /**
     * Return notif templates data
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_notif_templates(): array
    {
        $notif_templates=[
            ['id'=>'template1','file_name'=>'template1.jpg'],
            ['id'=>'template2','file_name'=>'template2.jpg','pro'=>true],
            ['id'=>'template3','file_name'=>'template3.jpg'],
            ['id'=>'template4','file_name'=>'template4.jpg','pro'=>true],
        ];
        return $notif_templates;
    }

    /**
     * Return notif tags data
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_notif_tags(): array
    {
        $tags = [
            ['text'=>'{first_name}','description'=>esc_html__("Displays the first name of the customer who placed the order.",Notifal_DOMAIN)],
            ['text'=>'{last_name}','description'=>esc_html__('Displays the last name of the customer who placed the order.',Notifal_DOMAIN)],
            ['text'=>'{order_time}','description'=>esc_html__('Displays the time elapsed since the order was placed, expressed dynamically in the most appropriate unit (e.g., seconds, minutes, hours, days).Examples: "Just now" (for recent orders), "15 minutes ago", "2 hours ago", "3 days ago"',Notifal_DOMAIN)],
            ['text'=>'{orders_counter}','description'=>esc_html__("Displays the total number of orders placed within the time range specified in the source section.Note: The time range can be set using the 'Orders from the last ... days' field in the source section. For example, if set to 7 days, this tag will show the total orders received in the past week.",Notifal_DOMAIN),'Pro'=>true],
            ['text'=>'{order_city}','description'=>esc_html__("Displays the city associated with the customer's order.",Notifal_DOMAIN)],
            ['text'=>'{order_state}','description'=>esc_html__("Displays the state associated with the customer's order.Note:This tag is exclusively applicable to real orders and does not support fake orders.",Notifal_DOMAIN)],
            ['text'=>'{order_country}','description'=>esc_html__("Displays the country associated with the customer's order.Note:This tag is exclusively applicable to real orders and does not support fake orders.",Notifal_DOMAIN)],
            ['text'=>'{order_company}','description'=>esc_html__('Displays the company name provided in the order details (if applicable).Note:This tag is exclusively applicable to real orders and does not support fake orders.',Notifal_DOMAIN)],
            ['text'=>'{product_name}','description'=>esc_html__("Displays the name of the product purchased in the order.",Notifal_DOMAIN)],
            ['text'=>'{product_stock}','description'=>esc_html__("Displays the current stock quantity of the product. Useful for creating urgency when stock is limited.",Notifal_DOMAIN)],
            ['text'=>'{product_link}','description'=>esc_html__("Generates a clickable link to the product page.Useful when use <a> tag in your texts.",Notifal_DOMAIN)],
            ['text'=>'{product_discount_amount}','description'=>esc_html__("Displays the discount amount applied to the product (e.g., $10 off).",Notifal_DOMAIN),'Pro'=>true],
            ['text'=>'{product_discount_percent}','description'=>esc_html__('Displays the discount percentage applied to the product (e.g., 25% off).',Notifal_DOMAIN),'Pro'=>true],
            ['text'=>'{product_sale_price}','description'=>esc_html__("Shows the product's current sale price.",Notifal_DOMAIN),'Pro'=>true],
            ['text'=>'{product_regular_price}','description'=>esc_html__("Displays the product's regular price before any discounts.",Notifal_DOMAIN),'Pro'=>true],
            ['text'=>'{product_sold_quantity}','description'=>esc_html__('Displays the total quantity of a product sold within the time period specified in the source section.Note:This tag is exclusively applicable to real orders and does not support fake orders.',Notifal_DOMAIN),'Pro'=>true],
            ['text'=>'{discount_ends_in}','description'=>esc_html__("Displays a live countdown to the end of the discount period using the most relevant unit: days, hours, or minutes. For example:,'Only 2 days left until the discount ends!','Only 5 hours left until the discount ends!','Only 30 minutes left until the discount ends!'. The countdown dynamically updates in real-time to encourage quick action.Note: Ensure you schedule the sale and set the 'Sale Price Dates' in the product settings for this tag to work effectively.",Notifal_DOMAIN),'Pro'=>true],
            ['text'=>'{copy_text text="Copy Code" copy_text="Enter text to copy"}','description'=>esc_html__("This tag creates a clickable text element that copies specific content to the visitor’s clipboard. The copy_text attribute defines the content to be copied, while the text attribute specifies the visible text that visitors click on to trigger the action.",Notifal_DOMAIN)]
        ];
        return $tags;
    }

    /**
     * Return all notif tags
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_all_tags(): array
    {
        $notif_tags = self::get_notif_tags();
        $all_tags = [];
        foreach ($notif_tags as $tags){
            $all_tags[] = trim($tags['text']);
        }
        return $all_tags;
    }



    /**
     * Return notif order tags
     *
     * @return string[]
     *@since 1.0.0
     */
    public static function require_orders_tag(): array
    {
        $tags = ['{first_name}','{last_name}','{order_state}','{order_country}',
            '{order_time}','{orders_counter}',
            '{order_city}','{order_company}','{product_sold_quantity}'];
        return $tags;
    }


    /**
     * Return notif stock tags
     *
     * @return string[]
     *@since 1.0.0
     */
    public static function stock_tags(): array
    {
        $tags = ['{product_stock}'];
        return $tags;
    }


    /**
     * Return notif discount tags
     *
     * @return string[]
     *@since 1.0.0
     */
    public static function discount_tags(): array
    {
        $tags = ['{product_discount_amount}',
                '{product_discount_percent}',
                '{product_sale_price}',
            '{discount_ends_in}'];
        return $tags;
    }

    /**
     * Return notif pro tags
     *
     * @return string[]
     *@since 1.0.0
     */
    public static function pro_tags(): array
    {
        $tags = [
            '{product_regular_price}',
            '{orders_counter}',
            '{product_sold_quantity}',
            '{product_discount_amount}',
            '{product_discount_percent}',
            '{product_sale_price}',
            '{discount_ends_in}'];
        return $tags;
    }

    /**
     * Return notif page display options
     *
     * @since 1.0.0
     * @return array[]
     */

    public static function page_display_options(): array
    {
        $options=[
            ['value'=>'page','label'=>esc_html__('Page',Notifal_DOMAIN)],
            ['value'=>'session','label'=>esc_html__('Session',Notifal_DOMAIN)],
        ];
        return $options;
    }


    /**
     * Return notif element display options
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_display_options(): array
    {
        $display_options=[
            ['id'=>'allow-close' , 'label'=>esc_html__('Close Button',Notifal_DOMAIN) , 'parent_id'=>'' , 'checked'=>'checked'],
            ['id'=>'first-additional-info' , 'label'=>esc_html__('Additional Info 1',Notifal_DOMAIN),'parent_id'=>'' , 'checked'=>'checked'],
            ['id'=>'second-additional-info' , 'label'=>esc_html__('Additional Info 2',Notifal_DOMAIN),'parent_id'=>'' , 'checked'=>'checked'],
            ['id'=>'action-button' , 'label'=>esc_html__('Action Button',Notifal_DOMAIN),'parent_id'=>'' , 'checked'=>'checked'],
            ['id'=>'if-close-notif' , 'label'=>esc_html__('Stop this Notif if user closed.',Notifal_DOMAIN),'parent_id'=>'if-close-notif-holder' , 'checked'=>''],
        ];
        if(maintenance::is_woocommerce_active()){
            $display_options[]= ['id'=>'product-image' , 'label'=>esc_html__('Product Image',Notifal_DOMAIN),'parent_id'=>'' , 'checked'=>'checked'];
        }
        return $display_options;
    }

    /**
     * get color defaults by template name,
     * if template is empty return all defaults
     *
     * @param  string $template
     * @return string[]|string[][]
     */
    public static function get_color_defaults_by_template(string $template=''): array
    {
        $color_defaults=[
            'template1'=>[
                'background_color' =>'#F6F6F6',
                'color2' =>'#020F84',
                'close_icon_color' => '#3c434a',
                'text_color' => '#6f6f6f',
                'first_additional_info_text_color' => '#6f6f6f',
                'second_additional_info_text_color' => '#6f6f6f',
                'action_btn_text_color' => '#ffffff',
                'action_btn_background_color' =>'#020F84',
            ],
            'template2'=>[
                'background_color' =>'#FFE3F1',
                'color2' =>'#FF0080',
                'close_icon_color' => '#3c434a',
                'text_color' => '#6f6f6f',
                'first_additional_info_text_color' => '#6f6f6f',
                'second_additional_info_text_color' => '#6f6f6f',
                'action_btn_text_color' => '#ffffff',
                'action_btn_background_color' =>'#FF2C95',
            ],
            'template3'=>[
                'background_color' =>'#ffffff',
                'color2' =>'#2454ff',
                'close_icon_color' => '#3c434a',
                'text_color' => '#6f6f6f',
                'first_additional_info_text_color' => '#6f6f6f',
                'second_additional_info_text_color' => '#6f6f6f',
                'action_btn_text_color' => '#ffffff',
                'action_btn_background_color' =>'#2454ff',
            ],
            'template4'=>[
                'background_color' =>'#FFC700',
                'color2' =>'',
                'close_icon_color' => '#3c434a',
                'text_color' => '#000',
                'first_additional_info_text_color' => '#6f6f6f',
                'second_additional_info_text_color' => '#6f6f6f',
                'action_btn_text_color' => '#ffffff',
                'action_btn_background_color' =>'#DA9224',
            ],
        ];
        if(empty($template)){
            return $color_defaults;
        }
        return $color_defaults[$template];
    }

    /**
     * Return notif color options
     *
     * @since 1.0.0
     * @return string[]
     */
    public static function get_color_options(): array
    {
        $color_options=[
            ['id'=>'background-color' , 'label'=>esc_html__('Background color',Notifal_DOMAIN)],
            ['id'=>'color2' , 'label'=>esc_html__('Color 2',Notifal_DOMAIN)],
            ['id'=>'close-icon-color' , 'label'=>esc_html__('Close icon',Notifal_DOMAIN)],
            ['id'=>'text-color' , 'label'=>esc_html__('Text',Notifal_DOMAIN)],
            ['id'=>'first-additional-info-text-color' , 'label'=>esc_html__('Additional Info 1',Notifal_DOMAIN)],
            ['id'=>'second-additional-info-text-color' , 'label'=>esc_html__('Additional Info 2',Notifal_DOMAIN)],
            ['id'=>'action-btn-text-color' , 'label'=>esc_html__('Action button text',Notifal_DOMAIN)],
            ['id'=>'action-btn-background-color' , 'label'=>esc_html__('Action button background',Notifal_DOMAIN)],
        ];
        return $color_options;
    }

    /**
     * Return notif position options
     *
     * @since 1.0.0
     * @return string[]
     */
    public static function get_notif_positions(): array
    {
        $notif_positions=[
            ['value'=>'bottom-left' , 'label'=>esc_html__('Bottom - Left',Notifal_DOMAIN)],
            ['value'=>'bottom-right' , 'label'=>esc_html__('Bottom - Right',Notifal_DOMAIN)],
            ['value'=>'top-left' , 'label'=>esc_html__('Top - Left',Notifal_DOMAIN)],
            ['value'=>'top-right' , 'label'=>esc_html__('Top - Right',Notifal_DOMAIN)],
            ['value'=>'top-center' , 'label'=>esc_html__('Top - Center',Notifal_DOMAIN)],
            ['value'=>'bottom-center' , 'label'=>esc_html__('Bottom - Center',Notifal_DOMAIN)],

        ];
        return $notif_positions;
    }

    /**
     * Return notif position distance options
     *
     * @since 1.0.0
     * @return string[]
     */
    public static function get_notif_position_distance(): array
    {
        $notif_position_distance=[
            ['id'=>'distance-top' , 'label'=>esc_html__('Distance From Top',Notifal_DOMAIN)],
            ['id'=>'distance-bottom' , 'label'=>esc_html__('Distance From Bottom',Notifal_DOMAIN)],
            ['id'=>'distance-left' , 'label'=>esc_html__('Distance From Left',Notifal_DOMAIN)],
            ['id'=>'distance-right' , 'label'=>esc_html__('Distance From Right',Notifal_DOMAIN)],
        ];
        return $notif_position_distance;
    }

    /**
     * Return notif fadein transitions options
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_fadein_transition_types(): array
    {
        $transition_types=[
            ['value'=>'fade_in_transition' , 'text'=>esc_html__('Fade in',Notifal_DOMAIN)],
            ['value'=>'slide_in_right_transition' , 'text'=>esc_html__('Slide in form right',Notifal_DOMAIN)],
            ['value'=>'slide_in_left_transition' , 'text'=>esc_html__('Slide in form left',Notifal_DOMAIN)],
            ['value'=>'slide_in_up_transition' , 'text'=>esc_html__('Slide in form up',Notifal_DOMAIN)],
            ['value'=>'slide_in_bottom_transition' , 'text'=>esc_html__('Slide in form bottom',Notifal_DOMAIN)],
        ];
        return $transition_types;
    }

    /**
     * Return notif fadeout transitions options
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_fadeout_transition_types(): array
    {
        $transition_types=[
            ['value'=>'fade_out_transition' , 'text'=>esc_html__('Fade out',Notifal_DOMAIN)],
            ['value'=>'slide_out_right_transition' , 'text'=>esc_html__('Slide out to right',Notifal_DOMAIN)],
            ['value'=>'slide_out_left_transition' , 'text'=>esc_html__('Slide out to left',Notifal_DOMAIN)],
            ['value'=>'slide_out_up_transition' , 'text'=>esc_html__('Slide out to up',Notifal_DOMAIN)],
            ['value'=>'slide_out_bottom_transition' , 'text'=>esc_html__('Slide out to bottom',Notifal_DOMAIN)],
        ];
        return $transition_types;
    }

    /**
     * Return notif delay options
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_delay_options(): array
    {
        $delay_options=[
            ['id'=>'display-duration' , 'label'=>esc_html__('Display Duration (seconds)',Notifal_DOMAIN) , 'value'=>5 , 'tooltip'=>esc_html__('Specify how long each Notif remains visible on the screen before it hides and prepares for the next one (if applicable).',Notifal_DOMAIN)],
            ['id'=>'initial-delay' , 'label'=>esc_html__('Initial Delay (seconds)',Notifal_DOMAIN),'value'=>10 , 'tooltip'=>esc_html__('Set how long to wait after a user arrives on the page before the first Notif is displayed.',Notifal_DOMAIN)],
            ['id'=>'interval-delay' , 'label'=>esc_html__('Interval Delay (seconds)',Notifal_DOMAIN),'value'=>10 , 'tooltip'=>esc_html__('Define the time interval between Notifs, ensuring they appear at a comfortable pace and don’t overwhelm users.',Notifal_DOMAIN)],
        ];
        return $delay_options;
    }


    /**
     * Return notif audio data
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_audio_data(){
        $audios=[
            ['value'=>'audio1','label'=>esc_html__('Audio 1',Notifal_DOMAIN) ],
            ['value'=>'audio2','label'=>esc_html__('Audio 2',Notifal_DOMAIN) ],
            ['value'=>'audio3','label'=>esc_html__('Audio 3',Notifal_DOMAIN) ],
            ['value'=>'audio4','label'=>esc_html__('Audio 4',Notifal_DOMAIN) ],
            ['value'=>'audio5','label'=>esc_html__('Audio 5',Notifal_DOMAIN) ],
            ['value'=>'custom-audio','label'=>esc_html__('Custom',Notifal_DOMAIN) ],
        ];
        return $audios;
    }

    /**
     * Return notif page display options
     *
     * @since 1.0.0
     * @return array[]
     */
    public static function get_page_conditions(){
        $conditions=[
            ['value'=>'all-pages','label'=>esc_html__('All Pages',Notifal_DOMAIN) ],
            ['value'=>'all-pages-except','label'=>esc_html__(' All Pages Except',Notifal_DOMAIN),'Pro'=>true],
            ['value'=>'specific-pages','label'=>esc_html__(' Specific Pages',Notifal_DOMAIN),'Pro'=>true],
        ];
        return $conditions;
    }


    /**
     * Return notif default text
     * if $type is not provided return main default text
     *
     * @since 1.0.0
     * @param $type
     * @return string
     */
    public static function get_default_notif_text($type='main'){
        switch ($type){
            case 'main':
                if(maintenance::is_woocommerce_active()){
                    $text = sprintf(esc_html__('%1s from %2s purchased %3s',Notifal_DOMAIN),'<b>{first_name} {last_name}</b>','<b>{order_city}</b>','<b>{product_name}</b>');
                }else{
                    $text = esc_html__('Join our growing community and stay updated with the latest news!',Notifal_DOMAIN);
                }
                break;
            case 'action_button':

                if(maintenance::is_woocommerce_active()){
                $text = sprintf(esc_html__('%1s Buy Now! %2s',Notifal_DOMAIN),'<a class="action-button" href="{product_link}">','</a>');
                }else{
                    $link = home_url();
                    $text = sprintf(esc_html__('%1s Join Now! %2s',Notifal_DOMAIN),'<a class="action-button" href="'.$link.'">','</a>');
                }
                break;
            case 'second_additional_info':
                if(maintenance::is_woocommerce_active()){
                $text = sprintf(esc_html__('Only %1s in stock',Notifal_DOMAIN),'<b>{product_stock}</b>');
                }else{
                    $text = esc_html__('Exclusive content',Notifal_DOMAIN);
                }
                break;
            case 'first_additional_info':
                if(maintenance::is_woocommerce_active()){
                $text = sprintf(esc_html__('%s ago',Notifal_DOMAIN),'{order_time}');
                }else{
                    $text = esc_html__('+5,000 joined!',Notifal_DOMAIN);
                }
                break;
        }
        return $text;
    }
}

