<?php
namespace Notifal;

use Notifal\notif\settings;

if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Templates handler
 *
 * @version 1.0.0
 */
class renderHtml{

    /**
     * print icon tooltip
     *
     * @param string $text - tooltip text
     * @param string $icon - tooltio icon class
     * @return void
     * @since 1.0.0
     */
    public static function icon_tooltip(string $text, string $icon='dashicons dashicons-info-outline'){
        ?>
        <div class="tooltip-container" data-description="<?php echo $text ?>">
            <span class="<?php echo $icon ?> pointer notif-text-tags tooltip-text" ></span>
        </div>
        <?php
    }

    /**
     * print 'checked' for input of checkbox
     *
     * @param $key
     * @param $value
     * @param $default
     * @param $conditions
     * @return void
     * @since 1.0.0
     */
    public static function toggle_checkbox($key,$value,$default='checked',$conditions=[]){
        if(notif::is_add_notif_page()) {
            if(empty($conditions)){
                echo $default;
            }else{
                $valid = true;
                foreach ($conditions as $key=>$value){
                    if($key != $value){
                        $valid = false;
                    }
                }
                if($valid){
                    echo $default;
                }

            }

        }else{
            if ($key === $value) {
                echo 'checked';
            }
        }
    }

    /**
     * print value $default_value on add new notif page otherwise print $setting_val
     *
     *
     * @param $setting_val
     * @param $default_value
     * @return void
     * @since 1.0.0
     */
    public static function input_value($setting_val,$default_value){
        if(notif::is_add_notif_page()) {
            $value = $default_value;
        }else{
            $value = $setting_val;
        }
        echo $value;
    }

    /**
     * print 'selected' if $data=$value
     *
     *
     * @param $data
     * @param $value
     * @return void
     * @since 1.0.0
     */
    public static function select_option_value($data,$value){
        if(!notif::is_add_notif_page()){
            if($data == $value){
                echo 'selected';
            }
        }
    }

    /**
     * print notifal-d-none to hide elements based on notif settings
     *
     *
     * @param $data
     * @param $conditions
     * @param string $type
     * @param $not
     * @param $add_notif_conditions
     * @return void
     * @since 1.0.0
     */
    public static function toggle_display($data, $conditions,string $type="in_array",$not=true,$add_notif_conditions=[]){
        if(notif::is_add_notif_page()){
            if(empty($add_notif_conditions)){
                echo 'notifal-d-none';
            }else{
                $check_add_conditions = true;
                foreach ($add_notif_conditions as $key=>$value){
                    if($not){
                        if(is_array($value)){
                            if(!in_array($key,$value)){
                                $check_add_conditions = false;
                                break;
                            }
                        }else{
                            if($key!=$value){
                                $check_add_conditions = false;
                                break;
                            }
                        }

                    }else{
                        if($key==$value){
                            $check_add_conditions = false;
                            break;
                        }
                    }

                }
                if($check_add_conditions){
                    echo 'notifal-d-none';
                }

            }

        }else{
            if($type=="in_array" && is_array($conditions)){
                if($not){
                    if(!in_array($data,$conditions)){
                        echo 'notifal-d-none';
                    }
                }else{
                    if(in_array($data,$conditions)){
                        echo 'notifal-d-none';
                    }
                }
            }else if($type="equal" && !is_array($conditions)){
                if($not) {
                    if($data != $conditions){
                        echo 'notifal-d-none';
                    }
                }else{
                    if($data == $conditions){
                        echo 'notifal-d-none';
                    }

                }

            }


        }

    }


    /**
     * Render backend notif texts
     *
     * @param $key
     * @param $add_notif_text
     * @param $html
     * @param $notif
     * @return void
     * @since 1.0.0
     */
    public static function render_backend_notif_text($key,$add_notif_text,$html=false,$notif=''){
        if(notif::is_add_notif_page()):
            echo self::replace_copy_text_tag($add_notif_text);
        else:
            if(is_admin()){
                echo self::replace_copy_text_tag(stripslashes(notif::get_notif_settings($key,$html)));
            }else{
                echo self::replace_copy_text_tag(stripslashes(notif::get_notif_settings($key,$html,$notif)));

            }
        endif;
    }

    /**
     * add class if $data and $value are equal or not
     *
     * @param $data
     * @param $value
     * @param $class
     * @param $not
     * @since 1.0.0
     * @return void
     */
    public static function add_class($data,$value,$class,$not=false){
        if(!$not){
            if($data==$value)
            echo $class;
        }else{
            if($data!==$value)
            echo $class;
        }
    }

    /**
     * Show product-category item on notif setting if any
     *
     *
     * @param $type
     * @param $prod_ids
     * @param $cat_ids
     * @since 1.0.0
     * @return void
     */
    public static function show_prod_cat_conditions($type,$prod_ids,$cat_ids){
        $notif_product_condition = settings::notif_products_condition();
        $type_name='';
        foreach ($notif_product_condition as $product_condition){
            if($product_condition['value']==$type){
                $type_name = $product_condition['name'];
            }
        }
        ?>
        <div class="notifal-tooltip-holder">
            <span class="notifal-tooltip-button"><?php echo $type_name?></span>
            <?php if($type=="specific-products" || $type=="all-products-except"): ?>
            <div id="selected-prod-cats" class="notifal-tooltip notifal-d-none">
                <?php
                if(!empty($prod_ids)):
                    ?>
                    <b><?php esc_html_e('Products:',Notifal_DOMAIN) ?></b>

                    <div class="selected-items">
                        <?php
                        foreach ($prod_ids as $id):
                        $product = wc_get_product($id);
                        $product_link=get_permalink($id);
                        $product_name= $product -> get_name();
                        ?>
                        <a target="_blank" href="<?php echo $product_link ?>"><?php echo $product_name . ' ' . '<span class="dashicons dashicons-admin-links"></span>' ?></a>
                        <?php
                    endforeach;
                    ?>
                    </div>
                        <?php
                endif;
                if(!empty($cat_ids)):
                    ?>
                    <b><?php esc_html_e('Categories:',Notifal_DOMAIN) ?></b>

                    <div class="selected-items">
                        <?php
                        foreach ($cat_ids as $id):
                        $term = get_term_by( 'id', $id, 'product_cat' );
                        $category_name=$term->name;
                        $category_slug = $term->slug;
                        $cat_link = get_term_link( $category_slug, 'product_cat' );;
                        ?>
                        <a target="_blank" href="<?php echo $cat_link ?>"><?php echo $category_name . ' ' . '<span class="dashicons dashicons-admin-links"></span>' ?></a>
                        <?php
                    endforeach; ?>
                    </div>
             <?php endif; ?>
                <span class="notifal-close-tooltip dashicons dashicons-no-alt"></span>
            </div>
            <?php endif; ?>
        </div>

        <?php
    }


    /**
     * Show page-category items if any
     *
     * @param $type
     * @param $page_ids
     * @param $cat_ids
     * @since 1.0.0
     * @return void
     */
    public static function show_page_conditions($type,$page_ids,$cat_ids){
        $notif_product_condition = settings::get_page_conditions();
        $type_name='';
        foreach ($notif_product_condition as $product_condition){
            if($product_condition['value']==$type){
                $type_name = $product_condition['label'];
            }
        }
        ?>
        <div class="notifal-tooltip-holder">
            <span class="notifal-tooltip-button"><?php echo $type_name?></span>
            <?php if($type=="all-pages-except" || $type=="specific-pages"): ?>
                <div id="selected-page" class="notifal-tooltip notifal-d-none">
                    <?php
                    if(!empty($page_ids)):
                        ?>
                        <b><?php esc_html_e('Pages:',Notifal_DOMAIN) ?></b>

                        <div class="selected-items">
                            <?php
                            foreach ($page_ids as $id):
                                $title = get_the_title($id);
                                $link=get_permalink($id);
                                ?>
                                <a target="_blank" href="<?php echo $link ?>"><?php echo $title . ' ' . '<span class="dashicons dashicons-admin-links"></span>' ?></a>
                            <?php
                            endforeach;
                            ?>
                        </div>
                    <?php
                    endif;
                    if(!empty($cat_ids)):
                        ?>
                        <b><?php esc_html_e('Categories:',Notifal_DOMAIN) ?></b>

                        <div class="selected-items">
                            <?php
                            foreach ($cat_ids as $id):
                                $term = get_term_by( 'id', $id, 'product_cat' );
                                $category_name=$term->name;
                                $cat_link = get_permalink($id);
                                ?>
                                <a target="_blank" href="<?php echo $cat_link ?>"><?php echo $category_name . ' ' . '<span class="dashicons dashicons-admin-links"></span>' ?></a>
                            <?php
                            endforeach; ?>
                        </div>
                    <?php endif; ?>
                    <span class="notifal-close-tooltip dashicons dashicons-no-alt"></span>
                </div>
            <?php endif; ?>
        </div>

        <?php
    }

    /**
     * render copy tag
     *
     * @param $content
     * @since 1.0.0
     * @return array|mixed|string|string[]|null
     */
    public static function replace_copy_text_tag($content) {
        if (strpos($content, 'copy_text') == false) return $content;
        $pattern = '/\{copy_text text="([^"]+)" copy_text="([^"]+)"\}/';

        $replacement = function($matches) {
            return '<span class="notif-copy-text" data-copy-text="' . esc_attr($matches[2]) . '">' . esc_html($matches[1]) . '</span>';
        };

        return preg_replace_callback($pattern, $replacement, $content);
    }

}