<?php
namespace Notifal;
use WC_Countries;
use WP_Query;

if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * WooCommerce handler
 * @version 1.0.0
 */
class woocommerce{

    use functions;

    public function __construct(){
        $ajax_data=[
            'search_prod_cat',
            'search_page_cat'
        ];
        $this->ajax_actions($this,$ajax_data);
    }


    /**
     * Get random product
     *
     * @param int $num
     * @param array $exclude
     * @param bool $sale_filter
     * @param bool $sale_end_date
     *
     * @since 1.0.0
     *
     * @return array
     */
    public function get_random_product(int $num=1, array $exclude=[], bool $sale_filter=false, bool $sale_end_date=false): array
    {
        if(maintenance::is_features_inactive()){
            return [];
        }
        $data=[];
        $meta_array=[];
        $args = [
            'post_type'   => ['product','product_variation'],
            'post_status' => 'publish',
            'posts_per_page' => $num,
            'orderby' => 'rand',
            'fields' => 'ids'
        ];
        if(maintenance::is_pro_active()){
            if(count($exclude)!==0){
                $args['post__not_in'] = $exclude;
            }
        }


        if($sale_filter){
            $meta_array[]= array(
                'key' => '_sale_price',
                'compare' => 'EXISTS',
            );
        }
        if($sale_end_date){
            $meta_array[]= array(
                'key' => '_sale_price_dates_to',
                'compare' => 'EXISTS',
            );
        }
        if(count($meta_array)!==0){
            $meta=array(
                'relation' => 'AND',
                $meta_array,
            );
            $args['meta_query'] = $meta ;
        }
        $query = new WP_Query( $args );
        $products = $query->get_posts();
        if(empty($products)) return [];
        $i=0;
        foreach ($products as $id){
            $product = wc_get_product($id);
            if($product->is_type( 'variable' )){
                $variation_ids = $product->get_children();
                $variation_id = array_rand($variation_ids);
                $product = wc_get_product($variation_ids[$variation_id]);
            }
            $data[$i]['id'] = $product->get_id();
            $data[$i]['name'] = $product->get_name();
            $data[$i]['stock'] = $product->get_stock_quantity();


            $data[$i]['image'] = $product -> get_image([80,80]);
            $data[$i]['product_link'] = $product -> get_permalink();

            $product_data = [];
            $product_data=apply_filters('Notifal_random_product_data',$product_data,$product);
            if(!empty($product_data)){
                foreach ($product_data as $key=>$value){
                    $data[$i][$key]=$value;
                }
            }

            $i++;


        }

        return $data;
    }


    /**
     * get order by notif
     *
     * @param object $notif
     *
     * @since 1.0.0
     *
     * @return array
     */
    public function get_orders_by_notif(object $notif) {
        $products_data = notif::get_notif_products('','',$notif);
        $order_threshold_num = notif::get_notif_settings('woo_orders_time',false,$notif);
        $if_show_order_time=notif::get_notif_settings('first_additional_info',false,$notif);
        $stock_max_quantity=notif::get_notif_settings('stock_max_quantity',false,$notif);
        $product_type='all-products';
        if(maintenance::is_pro_active()){
            if($products_data['type']==='all-products-except'){
                $product_type = 'all-products-except';
            }else if($products_data['type']==='specific-products'){
                $product_type = 'specific-products';
            }
        }


        $product_ids = notif::get_product_ids_by_notif($notif);
        $results=db::get_orders_by_products($product_type,$product_ids,$order_threshold_num);

        $orders_data  = array();
        if ( sizeof( $results )>0 ) {
            $j=0;
            foreach ( $results as $result ) {
                $order_product_ids=[];
                if(is_array($result)){
                    $order_id = $result['order_id'];
                }
                else{
                    $order_id=$result;
                }

                $order    = wc_get_order( $order_id );
                $orders_data [$j]['order_id'] =$order_id;
                $orders_data [$j]['first_name'] =$order->get_billing_first_name();
                $lastname=$order->get_billing_last_name();
                if(notif::get_notif_settings('first_letter_active',false,$notif) == 'true'){
                    $lastname=$this->make_abbravate($lastname);
                }
                $orders_data [$j]['last_name'] =$lastname;
                $orders_data [$j]['order_city'] = $order->get_billing_city();
                $orders_data [$j]['order_country'] = $this->change_country_format($order->get_billing_country());
                $orders_data [$j]['order_state'] = $this->change_state_format($order->get_billing_state(),$order->get_billing_country());
                $orders_data [$j]['order_company'] = $order->get_billing_company();
                if($if_show_order_time == 'true'){
                    $orders_data [$j]['order_time']=$this->time_substract($order->get_date_created()->date('Y-m-d H:i:s'));
                }
                if(empty($product_ids) || $products_data['type']=='all-products-except') {
                    foreach ($order->get_items() as $key => $lineItem) {
                        $order_product_ids[] = $lineItem['product_id'];
                    }
                }else{
                    $order_product_ids = $product_ids;
                }
                $rand_product_id = $order_product_ids[array_rand($order_product_ids)];
                $product= wc_get_product($rand_product_id);
                $orders_data[$j]['product_id'] = $product->get_id();
                $orders_data[$j]['product_name'] = $product->get_name();

                $orders_data[$j]['order_product_ids'] = $order_product_ids;
                if(maintenance::is_pro_active()){
                    $orders_data[$j]['product_sold_quantity'] = \Notifal_Pro\woocommerce::get_product_sold_quantity($product->get_id(),$order_threshold_num);
                }

                $orders_data[$j]['product_image'] = $product->get_image([80,80]);
                if($product->get_stock_quantity()<$stock_max_quantity && $product->get_stock_quantity()>0){
                    $orders_data[$j]['product_stock'] = $product->get_stock_quantity();
                }
                $orders_data[$j]['product_link'] = $product->get_permalink();
                if(maintenance::is_pro_active()){
                    $product_regular_price = $orders_data[$j]['product_regular_price'] = $product->get_regular_price();
                    $product_sales_price = $orders_data[$j]['product_sale_price'] = $product->get_sale_price();
                    if(!empty($product->get_sale_price())){
                        $discount_amount = $product_regular_price - $product_sales_price;
                        $discount_percent = floor(($discount_amount / $product_regular_price) * 100).'%';
                        $orders_data[$j]['product_discount_amount'] = wc_price((int)$discount_amount);
                        $orders_data[$j]['product_discount_percent'] = (int)$discount_percent.'%';

                    }
                    if(!empty($product->get_date_on_sale_to())){
                        $orders_data[$j]['discount_ends_in'] = $this->time_substract($product->get_date_on_sale_to()->date('Y-m-d H:i:s'),true);;
                    }
                }

                $j++;
            }
        }
        if(maintenance::is_features_inactive()){
            return [];
        }
        $orders_data = array_reverse($orders_data);
        return $orders_data;
    }

    /**
     * Change state code to state name
     *
     * @param $state_code
     * @param $country_code
     *
     * @since 1.0.0
     *
     * @return mixed
     */
    public static function change_state_format($state_code,$country_code){
        $countries = new WC_Countries();
        $country_states = $countries->get_states($country_code);
        $state_name     = $country_states[$state_code] ?? $state_code;
        return $state_name;
    }

    /**
     * change country code to country name
     *
     * @param $country_code
     *
     * @return mixed|string
     *
     * @since 1.0.0
     */
    public static function change_country_format($country_code){
        $countries =  WC()->countries->get_countries();
        return $countries[$country_code] ?? '';
    }


}