<?php
namespace Notifal;
use Notifal\notif as notif;
use Notifal\notif\analytics;
use Notifal\maintenance as maintenance;
use Notifal\features as features;
use Notifal\wordpress as wordpress;
use Notifal\backend\menu as menu;



if ( ! defined( 'ABSPATH' ) ) exit;

class plugin{


    /**
     * Notifications handler
     *
     * @since 1.0.0
     * @var notif
     */
    public notif $notif;


    /**
     * backend menu handler
     *
     * @since 1.0.0
     * @var menu
     */
    public menu $menu;

    /**
     * maintenance handler
     *
     * @since 1.0.0
     * @var maintenance
     */
    public maintenance $maintenance;


    /**
     * woocommerce handler
     *
     * @since 1.0.0
     * @var woocommerce
     */
    public woocommerce $woocommerce;

    /**
     * wordpress handler
     *
     * @since 1.0.0
     * @var wordpress
     */
    public wordpress $wordpress;


    /**
     * features handler
     *
     * @version 1.0.0
     * @var features
     */
    public features $features;

    /**
     * analytics handler
     *
     * @version 1.0.0
     * @var analytics
     */
    public analytics $analytics;



    /**
     * plugin instance
     * @var null
     */
    public static $instance = null;




    /**
     * Plugin constructor
     * initialize the plugin
     */
    public function __construct()
    {
        $this->register_autoloader();
        maintenance::init();
        add_action('init',[$this,'init_plugin'],99);
    }

    /**
     * Register Autoloader
     *
     * Autoload all classes and traits
     *
     * @since 1.0.0
     * @return void
     * @access private
     */
    private function register_autoloader(){
        require_once Notifal_INC . 'autoloader.php';
        autoloader::run();
    }


    /**
     * init component
     * @return void
     */
    public function init_plugin(){
        $this->init_components();
    }

    /**
     * init components
     *
     * init the plugin classes and components
     *
     * @since 1.0.0
     * @return void
     * @access private
     */
    private function init_components(){
        $this->maintenance = new maintenance();
        $this->notif = new notif();
        $this->analytics = new analytics();
        $this->wordpress = new wordpress();
        $this->menu = new menu();
        if(maintenance::is_woocommerce_active()){
            $this->woocommerce = new woocommerce();
        }
        $this->features = new features();
    }


    /**
     * Clone
     * Disable class cloning and throw an error on object clone.
     *
     */
    public function __clone() {
        _doing_it_wrong( __FUNCTION__, esc_html__( 'Something went wrong.', Notifal_DOMAIN ), '1.0.0' );
    }

    /**
     * Wakeup
     * Disable unserializing of the class.
     *
     */
    public function __wakeup() {
        _doing_it_wrong( __FUNCTION__, esc_html__( 'Something went wrong.', Notifal_DOMAIN ), '1.0.0' );
    }


    /**
     * get instance of class
     *
     * @return Plugin instance of the class.
     */

    public static function get_instance(){
        if ( is_null( self::$instance ) ) {
            self::$instance = new self();
        }

        return self::$instance;
    }



}
Plugin::get_instance();