<?php
namespace Notifal;

/**
 * General functions
 * @since 1.0.0
 *
 */

trait functions {

    /**
     * localize ajax scripts
     *
     * @param string $handle
     * @param array $objects
     *
     * @since 1.0.0
     *
     * @return void
     */
    public static function localize_ajax_scripts(string $handle,array $objects): void
    {
        if(empty($objects)) return;
        foreach ($objects as $item){
            wp_localize_script($handle, $item, array( 'ajax_url'  => admin_url('admin-ajax.php'), ));
        }
    }

    /**
     *
     * localize data scripts
     *
     * @param string $handle
     * @param string $objects_name
     * @param array $data
     *
     * @since 1.0.0
     *
     * @return void
     */
    public static function localize_data_scripts(string $handle,string $objects_name,array $data): void
    {
        wp_localize_script($handle, $objects_name, $data );

    }


    /**
     * get icon url
     *
     * @since 1.0.0
     *
     * @param $image
     * @return string
     */
    public static function get_image($image): string
    {
        return Notifal_URL . 'assets/images/'.$image;
    }


    /**
     * add action for ajax
     *
     * @param $class
     * @param array $ajax_data
     * @since 1.0.0
     * @return void
     */
    public static function ajax_actions($class,array $ajax_data){
        foreach ($ajax_data as $ajax){
            add_action('wp_ajax_'.$ajax, [$class, $ajax]);
        }
    }
    /**
     * add action for ajax - no private
     *
     * @param $class
     * @param array $ajax_data
     * @since 1.0.0
     * @return void
     */
    public static function noprivate_ajax_actions($class,array $ajax_data){
        foreach ($ajax_data as $ajax){
            add_action('wp_ajax_nopriv_'.$ajax, [$class, $ajax]);
            add_action('wp_ajax_'.$ajax, [$class, $ajax]);
        }
    }

    /**
     * Time subtraction
     *
     * @param $time
     * @param bool $number
     * @param bool $calculate
     *
     * @since 1.0.0
     *
     * @return false|string
     */
    public static function time_substract($time, $future = false, bool $number = false, bool $calculate = false, $only_day=false ) {
        if(maintenance::is_features_inactive()){
            return false;
        }
        if ( ! $number ) {
            if ( $time ) {
                $time = strtotime( $time );
            } else {
                return false;
            }
        }

        if ( ! $calculate ) {
            $current_time = current_time( 'timestamp' );
            if(!$future){
                $time_substract = $current_time - $time;
            }else{
                $time_substract = $time - $current_time;
            }

        } else {
            $time_substract = $time;
        }
        if ( $time_substract > 0 ) {

            $day = $time_substract / ( 24 * 3600 );
            $day = intval( $day );
            if($only_day){
                return $day;
            }
            if ( $day > 0 ) {
                if($day===1){
                    $day_text = esc_html__('day',Notifal_DOMAIN);
                }else{
                    $day_text = esc_html__('days',Notifal_DOMAIN);

                }
                return $day . ' ' . $day_text;
            }

            $hour = $time_substract / ( 3600 );
            $hour = intval( $hour );
            if ( $hour >= 1 ) {
                if($hour==1){
                    $hour_text = esc_html__('hour',Notifal_DOMAIN);
                }else{
                    $hour_text = esc_html__('hours',Notifal_DOMAIN);
                }
                return $hour . ' ' . $hour_text;
            }

            $min = $time_substract / ( 60 );
            $min = intval( $min );
            if ( $min >= 1 ) {
                if($min==1){
                    $min_text = esc_html__('minute',Notifal_DOMAIN);
                }else{
                    $min_text = esc_html__('minutes',Notifal_DOMAIN);
                }
                return $min . ' ' . $min_text;
            }

            $second= intval( $time_substract );
            if($second==1){
                $second_text = esc_html__('second',Notifal_DOMAIN);
            }else{
                $second_text = esc_html__('seconds',Notifal_DOMAIN);
            }
            return intval( $time_substract ) . $second_text;

        } else {
            return esc_html__('Just seconds',Notifal_DOMAIN);
        }
    }


    /**
     * get random time between two dates
     *
     * @param $start_date
     * @param $end_date
     *
     * @since 1.0.0
     *
     * @return false|string
     */
    public static function generate_random_time($start_date,$end_date){
        $min = strtotime($start_date);
        $max = strtotime($end_date);
        $rand_data = rand($min, $max);
        return date('Y-m-d H:i:s', $rand_data);
    }

    /**
     * make string abbreviated
     *
     * @param $string
     *
     * @since 1.0.0
     * @return string
     */
    public static function make_abbravate($string): string
    {
        return substr($string, 0, 1). '.';
    }


    /**
     * generate $num random times from $days ago
     *
     * @param $num
     * @param $days
     *
     * @since 1.0.0
     * @return array
     *
     */
    public static function generate_random_times($num,$days): array
    {
        $times=[];
        for($i=0; $i<$num; $i++){
            $min_date_of_fake_orders = date('Y-m-d h:i:s', mktime(0, 0, 0, date("m") , date("d") - $days, date("Y")));
            $current_time = date('Y-m-d h:i:s');
            $date = self::generate_random_time($min_date_of_fake_orders,$current_time);
            $times[]=self::time_substract($date);

        }
        return $times;

    }



}